SystemOrganization addCategory: #'Logging-Tests'!
SystemOrganization addCategory: #'Logging-Core'!
SystemOrganization addCategory: #'Logging-Adapters'!

Object subclass: #Bouncer
	instanceVariableNames: 'whitelist blacklist defaultAllow'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Logging-Core'!

!Bouncer commentStamp: 'mtf 4/25/2011 00:42' prior: 0!
I divide objects into two groups: allow, and deny, based on a whitelist, blacklist, and default policy. A bouncer is a slang term for a guard at a club who does this to people who want in. Judges objects based on equality; use my subclass IdentityBouncer for identity-based judging!

!Bouncer methodsFor: 'accessing lists' stamp: 'mtf 4/27/2011 14:05'!
accept: anObject
	self allow: anObject! !

!Bouncer methodsFor: 'accessing lists' stamp: 'mtf 4/27/2011 14:05'!
acceptAll: anObject
	self allowAll: anObject! !

!Bouncer methodsFor: 'accessing default' stamp: 'mtf 4/27/2011 14:04'!
acceptByDefault
	defaultAllow := true! !

!Bouncer methodsFor: 'testing' stamp: 'mtf 4/27/2011 14:05'!
accepts: anObject
	^ self allows: anObject! !

!Bouncer methodsFor: 'accessing lists' stamp: 'mtf 4/24/2011 23:33'!
allow: anObject
	blacklist remove: anObject ifAbsent: [].
	whitelist add: anObject! !

!Bouncer methodsFor: 'accessing lists' stamp: 'mtf 4/24/2011 23:36'!
allowAll: aCollection
	aCollection do: [:ea | self allow: ea]! !

!Bouncer methodsFor: 'accessing default' stamp: 'mtf 4/24/2011 23:31'!
allowByDefault
	defaultAllow := true! !

!Bouncer methodsFor: 'testing' stamp: 'mtf 4/24/2011 23:39'!
allows: anObject
	(blacklist includes: anObject) ifTrue: [^ false].
	(whitelist includes: anObject) ifTrue: [^ true].
	^ defaultAllow! !

!Bouncer methodsFor: 'accessing' stamp: 'mtf 4/24/2011 23:30'!
blacklist
	^ blacklist! !

!Bouncer methodsFor: 'accessing' stamp: 'mtf 4/24/2011 23:30'!
blacklist: aSet
	blacklist := aSet asSet! !

!Bouncer methodsFor: 'accessing lists' stamp: 'mtf 4/24/2011 23:37'!
default: anObject
	whitelist remove: anObject ifAbsent: [].
	blacklist remove: anObject ifAbsent: [].
! !

!Bouncer methodsFor: 'accessing lists' stamp: 'mtf 4/24/2011 23:37'!
defaultAll: aCollection
	aCollection do: [:ea | self default: ea]! !

!Bouncer methodsFor: 'accessing' stamp: 'mtf 4/24/2011 23:29'!
defaultAllow
	"Answer the value of defaultAllow"

	^ defaultAllow! !

!Bouncer methodsFor: 'accessing' stamp: 'mtf 4/24/2011 23:30'!
defaultAllow: aBoolean
	defaultAllow := aBoolean! !

!Bouncer methodsFor: 'testing' stamp: 'mtf 4/24/2011 23:40'!
denies: anObject
	^ (self allows: anObject) not! !

!Bouncer methodsFor: 'accessing lists' stamp: 'mtf 4/24/2011 23:33'!
deny: anObject
	whitelist remove: anObject ifAbsent: [].
	blacklist add: anObject! !

!Bouncer methodsFor: 'accessing lists' stamp: 'mtf 4/24/2011 23:36'!
denyAll: aCollection
	aCollection do: [:ea | self deny: ea]! !

!Bouncer methodsFor: 'accessing default' stamp: 'mtf 4/24/2011 23:31'!
denyByDefault
	defaultAllow := false! !

!Bouncer methodsFor: 'testing' stamp: 'mtf 4/24/2011 23:40'!
includes: anObject
	^ self allows: anObject! !

!Bouncer methodsFor: 'initialize-release' stamp: 'mtf 4/24/2011 23:24'!
initialize
	whitelist := Set new.
	blacklist := Set new.
	defaultAllow := true.
! !

!Bouncer methodsFor: 'accessing lists' stamp: 'mtf 4/24/2011 23:34'!
reject: anObject
	self deny: anObject! !

!Bouncer methodsFor: 'accessing lists' stamp: 'mtf 4/24/2011 23:37'!
rejectAll: aCollection
	self denyAll: aCollection! !

!Bouncer methodsFor: 'accessing default' stamp: 'mtf 4/24/2011 23:31'!
rejectByDefault
	defaultAllow := false! !

!Bouncer methodsFor: 'testing' stamp: 'mtf 4/24/2011 23:40'!
rejects: anObject
	^ self denies: anObject! !

!Bouncer methodsFor: 'testing' stamp: 'mtf 4/24/2011 23:41'!
value: anObject
	^ self allows: anObject! !

!Bouncer methodsFor: 'accessing' stamp: 'mtf 4/24/2011 23:30'!
whitelist
	^ whitelist! !

!Bouncer methodsFor: 'accessing' stamp: 'mtf 4/24/2011 23:30'!
whitelist: aSet
	whitelist := aSet asSet! !

Bouncer subclass: #IdentityBouncer
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Logging-Core'!

!IdentityBouncer commentStamp: 'mtf 4/25/2011 00:41' prior: 0!
Like Bouncer, but accepts or rejects objects based on identity rather than equality!

!IdentityBouncer methodsFor: 'accessing' stamp: 'mtf 4/24/2011 23:43'!
blacklist: aSet
	blacklist := aSet asIdentitySet! !

!IdentityBouncer methodsFor: 'initialize-release' stamp: 'mtf 4/24/2011 23:42'!
initialize
	whitelist := IdentitySet new.
	blacklist := IdentitySet new.
	defaultAllow := true.
! !

!IdentityBouncer methodsFor: 'accessing' stamp: 'mtf 4/24/2011 23:43'!
whitelist: aSet
	whitelist := aSet asIdentitySet! !

Object subclass: #LogEngine
	instanceVariableNames: 'id filter'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Logging-Adapters'!

!LogEngine commentStamp: 'kph 12/12/2007 13:06' prior: 0!
A placeholder!

!LogEngine class methodsFor: 'as yet unclassified' stamp: 'kph 11/26/2008 06:19'!
LogClient: block 

	Smalltalk at: #LogClient ifPresent: block! !

!LogEngine class methodsFor: 'as yet unclassified' stamp: 'kph 12/1/2008 04:42'!
for: aName

	^ self basicNew setId: aName; initialize; yourself! !

!LogEngine class methodsFor: 'as yet unclassified' stamp: 'kph 11/26/2008 06:21'!
start

	self start.
	
	self LogClient: [ :sl |   ^ self ].

	^ self error: 'LogEngine is not loaded'! !

!LogEngine methodsFor: 'accessing' stamp: 'mtf 4/25/2011 01:03'!
filter
	^ filter! !

!LogEngine methodsFor: 'accessing' stamp: 'mtf 4/25/2011 01:04'!
filter: aValuable
	filter := aValuable! !

!LogEngine methodsFor: 'as yet unclassified' stamp: 'kph 11/30/2008 05:29'!
id

	^ id ifNil: [ self class name ]! !

!LogEngine methodsFor: 'as yet unclassified' stamp: 'mtf 4/29/2011 16:36'!
logEntry: aLogEntry formatted: aLogFormatter
"ignores formatter for now"
	
	| le levelSelector out |
 
	self class LogClient: [ :log | 
		
			le := log for: self id.
			
			levelSelector := aLogEntry level asMutator.
		
			out := String new writeStream << '[' << aLogEntry pid asString << '] ' << (aLogEntry category ifNil: ['']) << ' ' << aLogEntry line.
		
			le stamp: aLogEntry stamp.
			(le respondsTo: levelSelector) 
				ifTrue: [ le perform: levelSelector with: out contents ]
				ifFalse: [ le privateInfo: aLogEntry level asUppercase msg: out contents ].
	]
	
! !

!LogEngine methodsFor: 'as yet unclassified' stamp: 'kph 11/30/2008 05:28'!
setId: aName

	id := aName! !

!LogEngine methodsFor: 'as yet unclassified' stamp: 'mtf 4/29/2011 16:32'!
wouldLogEntry: aLogEntry
	filter ifNil: [^ true].
	^ filter value: aLogEntry! !

Object subclass: #LogEntry
	instanceVariableNames: 'properties stamp line level category sender pid source processDescription'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Logging-Core'!

!LogEntry commentStamp: 'mtf 4/29/2011 17:01' prior: 0!
an entry in a log file

properties <Dictionary>: arbitrary properties the formatter can display how it wants
stamp <DateAndTime>: the log entry timestamp
line <String>: the text of the log entry
level <Symbol>: the verbosity level of the message. Their meaning is defined in LogCurrent
category <Symbol>: a classification of this message, mostly for filtering purposes
sender <ContextPart>: The active stack frame that is making this entry
pid <Integer>: an identifier for the running process; default is "Processor activeProcess identityHash"
source <Object>: the object making this log entry; default is "sender reciever"
processDescription <String>: a description of what the process that made this entry is doing
!

LogEntry subclass: #AccessLogEntry
	instanceVariableNames: 'senderHost senderName userName status size'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Logging-Core'!

!AccessLogEntry commentStamp: 'mtf 4/25/2011 18:45' prior: 0!
a LogMessage for logging server accesses, in apache access_log format by default!

!AccessLogEntry methodsFor: 'printing' stamp: 'mtf 5/3/2011 18:42'!
logFormatted: aFormatter on: aStream
	aFormatter logAccessEntry: self on: aStream! !

!AccessLogEntry methodsFor: 'accessing' stamp: 'mtf 4/25/2011 18:59'!
request
	^ line! !

!AccessLogEntry methodsFor: 'accessing' stamp: 'mtf 4/25/2011 18:59'!
request: aString
	line := aString! !

!AccessLogEntry methodsFor: 'accessing' stamp: 'mtf 4/25/2011 17:04'!
senderHost
	^ senderHost! !

!AccessLogEntry methodsFor: 'accessing' stamp: 'mtf 4/25/2011 17:05'!
senderHost: aByteArray
	senderHost := aByteArray! !

!AccessLogEntry methodsFor: 'accessing' stamp: 'mtf 4/25/2011 17:16'!
senderHostString
	^ senderHost ifNotNil: [NetNameResolver stringFromAddress: senderHost]! !

!AccessLogEntry methodsFor: 'accessing' stamp: 'mtf 4/25/2011 17:05'!
senderHostString: aString
	senderHost := NetNameResolver addressFromString: aString! !

!AccessLogEntry methodsFor: 'accessing' stamp: 'mtf 4/25/2011 17:04'!
senderName
	"Answer the value of senderName"

	^ senderName! !

!AccessLogEntry methodsFor: 'accessing' stamp: 'mtf 4/25/2011 17:04'!
senderName: anObject
	"Set the value of senderName"

	senderName := anObject! !

!AccessLogEntry methodsFor: 'accessing' stamp: 'mtf 5/5/2011 19:28'!
size
	^ size! !

!AccessLogEntry methodsFor: 'accessing' stamp: 'mtf 4/25/2011 17:04'!
size: anObject
	"Set the value of size"

	size := anObject! !

!AccessLogEntry methodsFor: 'accessing' stamp: 'mtf 4/25/2011 17:04'!
status
	"Answer the value of status"

	^ status! !

!AccessLogEntry methodsFor: 'accessing' stamp: 'mtf 4/25/2011 17:04'!
status: anObject
	"Set the value of status"

	status := anObject! !

!AccessLogEntry methodsFor: 'accessing' stamp: 'mtf 4/25/2011 17:04'!
userName
	"Answer the value of userName"

	^ userName! !

!AccessLogEntry methodsFor: 'accessing' stamp: 'mtf 4/25/2011 17:04'!
userName: anObject
	"Set the value of userName"

	userName := anObject! !

LogEntry subclass: #ErrorLogEntry
	instanceVariableNames: 'error'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Logging-Core'!

!ErrorLogEntry commentStamp: 'mtf 5/3/2011 18:31' prior: 0!
I dump unhandled squeak errors to the log!

!ErrorLogEntry methodsFor: 'accessing' stamp: 'mtf 5/3/2011 18:33'!
error
	^ error! !

!ErrorLogEntry methodsFor: 'accessing' stamp: 'mtf 5/3/2011 18:32'!
error: anException
	error := anException! !

!ErrorLogEntry methodsFor: 'printing' stamp: 'mtf 5/3/2011 18:47'!
logFormatted: aFormatter on: aStream
	aFormatter logErrorEntry: self on: aStream! !

!LogEntry methodsFor: 'accessing' stamp: 'mtf 4/24/2011 19:25'!
category
	"Answer the value of category"

	^ category! !

!LogEntry methodsFor: 'accessing' stamp: 'mtf 4/24/2011 19:25'!
category: anObject
	"Set the value of category"

	category := anObject! !

!LogEntry methodsFor: 'accessing' stamp: 'mtf 4/24/2011 19:25'!
level
	"Answer the value of level"

	^ level! !

!LogEntry methodsFor: 'accessing' stamp: 'mtf 4/24/2011 19:25'!
level: anObject
	"Set the value of level"

	level := anObject! !

!LogEntry methodsFor: 'accessing' stamp: 'mtf 4/24/2011 19:25'!
line
	"Answer the value of line"

	^ line! !

!LogEntry methodsFor: 'accessing' stamp: 'mtf 4/24/2011 19:25'!
line: anObject
	"Set the value of line"

	line := anObject! !

!LogEntry methodsFor: 'printing' stamp: 'mtf 5/3/2011 18:41'!
logFormatted: aFormatter on: aStream
	aFormatter logTextEntry: self on: aStream! !

!LogEntry methodsFor: 'accessing' stamp: 'mtf 4/24/2011 20:07'!
numericLevel
	^ LogCurrent levelIndexAt: level.
! !

!LogEntry methodsFor: 'accessing' stamp: 'mtf 4/24/2011 19:25'!
pid
	"Answer the value of pid"

	^ pid! !

!LogEntry methodsFor: 'accessing' stamp: 'mtf 4/24/2011 19:25'!
pid: anObject
	"Set the value of pid"

	pid := anObject! !

!LogEntry methodsFor: 'accessing' stamp: 'mtf 4/29/2011 17:03'!
processDescription
	^ processDescription! !

!LogEntry methodsFor: 'accessing' stamp: 'mtf 4/29/2011 17:03'!
processDescription: aString
	processDescription := aString! !

!LogEntry methodsFor: 'accessing' stamp: 'mtf 4/24/2011 19:26'!
properties

	^ properties ifNil: [ properties := IdentityDictionary new ]! !

!LogEntry methodsFor: 'accessing' stamp: 'mtf 4/24/2011 19:25'!
sender
	"Answer the value of sender"

	^ sender! !

!LogEntry methodsFor: 'accessing' stamp: 'mtf 4/24/2011 19:25'!
sender: anObject
	"Set the value of sender"

	sender := anObject! !

!LogEntry methodsFor: 'accessing' stamp: 'mtf 5/5/2011 19:40'!
source
	source ifNotNil: [^ source].
	sender ifNil: [^ nil].
	^ sender receiver! !

!LogEntry methodsFor: 'accessing' stamp: 'mtf 4/29/2011 17:02'!
source: anObject
	source := anObject! !

!LogEntry methodsFor: 'accessing' stamp: 'mtf 4/24/2011 19:41'!
stamp
	"Answer the value of stamp"

	^ stamp ifNil: [stamp := DateAndTime now]! !

!LogEntry methodsFor: 'accessing' stamp: 'mtf 4/24/2011 19:25'!
stamp: anObject
	"Set the value of stamp"

	stamp := anObject! !

Object subclass: #LogFile
	instanceVariableNames: 'file filter'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Logging-Adapters'!

!LogFile commentStamp: 'gk 11/20/2007 22:39' prior: 0!
This is a simple emitter to a log file on disk to be used with SLLog. See #start to see how it is added to the SLLog singleton as an emitter. It handles log rotation with a max filesize and number of old logfiles kept around.

	SLLogFile open!

!LogFile class methodsFor: 'settings' stamp: 'gk 2/16/2007 11:19'!
baseName
	^'squeak'! !

!LogFile class methodsFor: 'settings' stamp: 'gk 11/13/2006 14:38'!
extension
	^'log'! !

!LogFile class methodsFor: 'private' stamp: 'gk 11/13/2006 14:47'!
filename
	^self baseName, '.', self extension ! !

!LogFile class methodsFor: 'settings' stamp: 'gk 6/1/2007 12:47'!
filesKept

	^5! !

!LogFile class methodsFor: 'instance creation' stamp: 'kph 12/2/2008 03:52'!
logDirectory

 	^ FileDirectory default pathName! !

!LogFile class methodsFor: 'settings' stamp: 'kph 12/2/2008 03:59'!
maxLevel

	^ 10! !

!LogFile class methodsFor: 'settings' stamp: 'gk 11/13/2006 15:03'!
maxSize

	^10000000 "10 MB"! !

!LogFile class methodsFor: 'instance creation' stamp: 'kph 2/24/2009 16:58'!
named: fileName 
	"Create a new log file and start logging"

	^self new open: fileName! !

!LogFile class methodsFor: 'instance creation' stamp: 'kph 12/12/2007 09:50'!
open
	"self open"

	self open: self filename! !

!LogFile class methodsFor: 'instance creation' stamp: 'kph 12/2/2008 03:53'!
open: filename
	"self open"

	self named: self logDirectory, FileDirectory slash, filename
	! !

!LogFile methodsFor: 'log protocol' stamp: 'kph 2/24/2009 16:36'!
critical: aBlock

	^ LogCurrent critical: aBlock! !

!LogFile methodsFor: 'private' stamp: 'gk 11/13/2006 15:02'!
deleteOldLogFilesFromNumber: toDelete

	| num dir fn |
	toDelete < 1 ifTrue: [^self].
	num := toDelete.
	dir := file directory.
	[fn := self class baseName, '.',num printString,'.', self class extension.
	dir fileExists: fn] whileTrue: [
		dir deleteFileNamed: fn.
		num := num - 1]! !

!LogFile methodsFor: 'accessing' stamp: 'mtf 4/25/2011 01:10'!
filter
	^ filter! !

!LogFile methodsFor: 'accessing' stamp: 'mtf 4/25/2011 01:10'!
filter: aValuable
	filter := aValuable! !

!LogFile methodsFor: 'private' stamp: 'gk 8/17/2006 17:13'!
flush
	file ensureOpen; flush! !

!LogFile methodsFor: 'log protocol' stamp: 'mtf 8/6/2011 19:00'!
logEntry: aLogEntry formatted: aLogFormatter
		
	self critical: [[
		(self wouldLogEntry: aLogEntry) ifFalse: [ ^ self ].
	
		self writeLine: (String streamContents: [:stream |
			aLogEntry logFormatted: aLogFormatter on: stream]).

		"Rotate log file if max size is reached."
		file position > self class maxSize ifTrue: [self rotate]
	] on: Error do: ["if I can't write to the file, silently give up. Mostly this happens when starting up the image on a different computer than the log file was on"]]! !

!LogFile methodsFor: 'private' stamp: 'mtf 6/3/2011 16:15'!
open: fileName
	file := AppendFileStream fileNamed: fileName! !

!LogFile methodsFor: 'printing' stamp: 'mtf 4/27/2011 13:55'!
printOn: aStream
	super printOn: aStream.
	file ifNil: [^ self].
	aStream nextPutAll: ' on '; nextPutAll: file localName! !

!LogFile methodsFor: 'log protocol' stamp: 'makl 8/11/2006 10:01'!
reInitialize
	"Do nothing"! !

!LogFile methodsFor: 'private' stamp: 'gk 8/7/2006 11:13'!
reset
	file reset! !

!LogFile methodsFor: 'log protocol' stamp: 'kph 2/2/2008 06:25'!
rotate
	"Close the current one. Maybe delete old log files, keeping last n.
	Rename the current with highest number + 1. Open a new file."

	| dir nextName number |
	file close.
	dir := file directory.
	nextName := dir nextNameFor: self class baseName extension: self class extension.
	number := (dir splitNameVersionExtensionFor: nextName) at: 2.
	self deleteOldLogFilesFromNumber: number - self class filesKept.
	[ dir rename: file name toBe: nextName ] ifError: [ ].
	self open: self class filename
	! !

!LogFile methodsFor: 'log protocol' stamp: 'mtf 4/29/2011 16:32'!
wouldLogEntry: aLogEntry
	filter ifNil: [^ true].
	^ filter value: aLogEntry! !

!LogFile methodsFor: 'private' stamp: 'mtf 6/3/2011 16:23'!
writeLine: aString
	aString ifNil: [^ self].
	file ifNil: [^ self].
	file critical: [file nextPutAll: aString, String crlf]! !

Object subclass: #LogFilter
	instanceVariableNames: 'maxNumericLevel categoryBouncer categoryFilter'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Logging-Core'!

!LogFilter methodsFor: 'accessing' stamp: 'mtf 4/25/2011 00:47'!
categoryFilter
	^ categoryFilter! !

!LogFilter methodsFor: 'initialize' stamp: 'mtf 4/25/2011 01:06'!
initialize
	maxNumericLevel := 11.
	categoryFilter := IdentityBouncer new.! !

!LogFilter methodsFor: 'accessing' stamp: 'mtf 4/24/2011 23:07'!
maxLevel: aSymbol
	self maxNumericLabel: (LogCurrent levelIndexAt: aSymbol)! !

!LogFilter methodsFor: 'accessing' stamp: 'mtf 4/25/2011 00:40'!
maxNumericLevel
	^ maxNumericLevel! !

!LogFilter methodsFor: 'accessing' stamp: 'mtf 4/24/2011 23:08'!
maxNumericLevel: anInteger
	maxNumericLevel := anInteger! !

!LogFilter methodsFor: 'testing' stamp: 'mtf 4/29/2011 16:34'!
shouldLogEntry: aLogEntry
	aLogEntry numericLevel <= maxNumericLevel ifFalse: [^ false].
	(categoryFilter allows: aLogEntry category) ifFalse: [^ false].
	^ true! !

!LogFilter methodsFor: 'testing' stamp: 'mtf 4/29/2011 16:35'!
value: aLogEntry
	^ self shouldLogEntry: aLogEntry! !

Object subclass: #LogFormatter
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Logging-Core'!

!LogFormatter methodsFor: 'error' stamp: 'mtf 5/3/2011 18:45'!
errorDumpFileName
	^ Smalltalk squeakErrorFileName! !

!LogFormatter methodsFor: 'access' stamp: 'mtf 5/3/2011 18:42'!
logAccessEntry: aMessage on: aStream
"writes in Apache Common Log Format: http://httpd.apache.org/docs/2.0/logs.html"

	aStream nextPutAll: (aMessage senderHostString ifNil: '-'); space.
	aStream nextPutAll: (aMessage senderName ifNil: '-'); space.
	aStream nextPutAll: (aMessage userName ifNil: '-'); space.
	self logDateApache: aMessage on: aStream. aStream space.
	aStream nextPut: $";
		nextPutAll: (aMessage request ifNil: ['-']);
		nextPutAll: '" '.
	aStream nextPutAll: (aMessage status ifNil: '-'); space.
	aMessage size ifNil: [aStream nextPutAll: '- '] ifNotNil: [aStream print: aMessage size].
! !

!LogFormatter methodsFor: 'text' stamp: 'mtf 4/27/2011 18:48'!
logDate: aMessage on: aStream
	aMessage stamp printLocalOn: aStream! !

!LogFormatter methodsFor: 'access' stamp: 'mtf 4/27/2011 18:53'!
logDateApache: aMessage on: aStream
"writes in Apache Common Log Format: http://httpd.apache.org/docs/2.0/logs.html"

	| stamp |
	stamp := aMessage stamp.
	aStream nextPut: $[;
		nextPutAll: (stamp day asString padded: #left to: 2 with: $0);
		nextPut: $/;
		nextPutAll: stamp monthAbbreviation;
		nextPut: $/;
		nextPutAll: (stamp year asString padded: #left to: 4 with: $0);
		nextPut: $:.

	stamp printHMSOn: aStream.

	aStream
		space;
		nextPut: (stamp offset positive ifTrue: [$+] ifFalse: [$-]);
		nextPutAll: (stamp offset hours abs asString padded: #left to: 2 with: $0);
		nextPutAll: (stamp offset minutes abs asString padded: #left to: 2 with: $0);
		nextPut: $]! !

!LogFormatter methodsFor: 'error' stamp: 'mtf 5/3/2011 19:03'!
logErrorEntry: aLogEntry on: aStream
	| filename |
	filename := self errorDumpFileName.
	Smalltalk dumpException: aLogEntry error to: filename.
	aLogEntry line: aLogEntry error description, '; stack written to ', filename.
	self logTextEntry: aLogEntry on: aStream! !

!LogFormatter methodsFor: 'text' stamp: 'mtf 4/27/2011 18:49'!
logSender: aMessage on: aStream
	"handle blocks differently to avoid decompile in 'sender asString' "
	aMessage sender isBlock
		ifTrue: [aStream << $[  << aMessage sender method methodClass name
			<< '<<' <<  aMessage sender methodSelector << $]]
		ifFalse: [aStream << $( << aMessage sender asString << $)].	! !

!LogFormatter methodsFor: 'text' stamp: 'mtf 4/29/2011 17:37'!
logSource: aLogEntry on: aStream
	aLogEntry source logOn: aStream! !

!LogFormatter methodsFor: 'text' stamp: 'mtf 5/5/2011 17:59'!
logTextEntry: aMessage on: aStream
	
	self logDate: aMessage on: aStream.
	
	aStream << ' - ' << aMessage level << '(' << aMessage numericLevel << ')' << (aMessage category ifNil: ['']) << ': ' << aMessage line << ' '. 

	aMessage processDescription ifNotNil: [aStream << aMessage processDescription << ', '].
	self logSource: aMessage on: aStream.
	self logSender: aMessage on: aStream.
! !

Object subclass: #LogHistory
	instanceVariableNames: 'history next mutex'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Logging-Adapters'!

!LogHistory class methodsFor: 'as yet unclassified' stamp: 'kph 11/30/2008 00:15'!
size: n

	^ self basicNew setSize: n; initialize; yourself! !

!LogHistory methodsFor: 'as yet unclassified' stamp: 'kph 12/1/2008 23:08'!
addToHistory: aRecord

	mutex critical: [
		
		history at: next put: aRecord.
		next := (history size = next) ifTrue: [ 1 ] ifFalse: [ next + 1 ].
	
	]! !

!LogHistory methodsFor: 'as yet unclassified' stamp: 'jrd 5/4/2009 23:31'!
clearEntries

	self setSize: history size.
	next := 1.! !

!LogHistory methodsFor: 'as yet unclassified' stamp: 'mtf 4/25/2011 01:10'!
filter
	^ nil! !

!LogHistory methodsFor: 'as yet unclassified' stamp: 'mtf 4/25/2011 01:10'!
filter: aValuable! !

!LogHistory methodsFor: 'as yet unclassified' stamp: 'kph 12/1/2008 23:08'!
initialize

	mutex := Semaphore forMutualExclusion! !

!LogHistory methodsFor: 'as yet unclassified' stamp: 'mtf 4/29/2011 16:37'!
logEntry: aLogEntry formatted: aLogFormatter

	self addToHistory: thisContext asMessage! !

!LogHistory methodsFor: 'as yet unclassified' stamp: 'kph 12/2/2008 02:49'!
outputTo: adapter

	mutex critical: [

		next to: history size do: [ :n | (history at: n) ifNotNilDo: [ :msg | msg sendTo: adapter ] ].
		1 to: next - 1 do: [ :n | (history at: n) ifNotNilDo: [ :msg |  msg sendTo: adapter ] ].
	
	].! !

!LogHistory methodsFor: 'as yet unclassified' stamp: 'kph 12/1/2008 02:26'!
postCopy

	history := history copy! !

!LogHistory methodsFor: 'as yet unclassified' stamp: 'kph 11/30/2008 00:13'!
setSize: size

	history := Array new: size.
	next := 1.! !

!LogHistory methodsFor: 'as yet unclassified' stamp: 'mtf 4/29/2011 16:32'!
wouldLogEntry: aLogEntry
	^ true! !

Object subclass: #LogReport
	instanceVariableNames: 'router item'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Logging-Core'!

!LogReport commentStamp: 'mtf 4/4/2009 00:55' prior: 0!
I am a convenience class for dumping the state of an object to the log. My instances know how to write useful, formatted information about an object and its class to the log. See the methods in my public category for what I can dump. To get an instance of me:

self log report: anObject	"a reporter on any object"
self log my					"a reporter on self"
self log it					"a reporter on self"

to dump the instance variables of self to the log:
self log my vars.
self log my slots.
self log my instance.

to dump the class variables of self:
self log my classVars.

to dump all the variables of self:
self log my all.!

!LogReport methodsFor: 'public' stamp: 'kph 11/27/2008 02:28'!
all
	self slots.
	item class class allInstVarNames isEmpty ifFalse: [ self classInstance ].
	item class classPool isEmpty ifFalse: [ self classVars ].
	item class sharedPools isEmpty ifFalse: [	self sharedPools ].
	

! !

!LogReport methodsFor: 'public' stamp: 'kph 12/3/2008 20:19'!
classInstance 
	router category: 'Class'.
	self headerFor: item class.
	self contentsOf: item class! !

!LogReport methodsFor: 'public' stamp: 'kph 12/3/2008 20:19'!
classPool
	self title: 'Class Variables'.
	router detailed: item class classPool! !

!LogReport methodsFor: 'public' stamp: 'kph 12/12/2007 17:58'!
classVars

	self classPool! !

!LogReport methodsFor: 'public' stamp: 'kph 11/27/2008 02:25'!
contents
	self title: 'Contents'.
	self contentsOf: item! !

!LogReport methodsFor: 'private' stamp: 'kph 12/3/2008 20:19'!
contentsOf: theItem 
	| isMeta |
	isMeta := theItem class isMeta.
	
	theItem class allInstVarNames keysAndValuesDo: [:index :each |
		index = 1 ifTrue: [ 
			self title: (isMeta ifTrue: [ 'class ' ] ifFalse: [ '' ]),'Named Slots' ].
		(router << each << ': ' << (theItem instVarAt: index)) cr.
	].		 
	(1 to: theItem basicSize) do: [:index |
		index = 1 ifTrue: [ self title: 'Indexed Slots' ].
		(router << index << ': ' << (theItem instVarAt: index)) cr.  
	].
	router endEntry.
	
! !

!LogReport methodsFor: 'private' stamp: 'kph 12/3/2008 20:17'!
headerFor: theItem

	router << theItem ; cr! !

!LogReport methodsFor: 'public' stamp: 'kph 11/27/2008 02:26'!
instance
 	self contentsOf: item! !

!LogReport methodsFor: 'accessing' stamp: 'kph 12/12/2007 17:25'!
item
	"Answer the value of item"

	^ item! !

!LogReport methodsFor: 'accessing' stamp: 'kph 12/12/2007 17:25'!
item: anObject
	"Set the value of item"

	item := anObject! !

!LogReport methodsFor: 'accessing' stamp: 'kph 11/29/2008 03:22'!
router
	"Answer the value of router"

	^ router! !

!LogReport methodsFor: 'accessing' stamp: 'kph 11/29/2008 03:22'!
router: anObject
	"Set the value of router"

	router := anObject! !

!LogReport methodsFor: 'public' stamp: 'kph 12/12/2007 17:57'!
shared

	^ self sharedPools! !

!LogReport methodsFor: 'public' stamp: 'kph 12/3/2008 20:18'!
sharedPools
	self title: 'Shared Pools'.
	router detailed: item class sharedPools
	

 ! !

!LogReport methodsFor: 'public' stamp: 'kph 12/3/2008 20:17'!
slots
	router category: 'Instance'.
	self contentsOf: item! !

!LogReport methodsFor: 'private' stamp: 'kph 12/3/2008 20:18'!
title: text

	router << text asUppercase ; cr! !

!LogReport methodsFor: 'public' stamp: 'kph 11/27/2008 02:26'!
vars
 	self contentsOf: item! !

Object subclass: #LogSyslogSender
	instanceVariableNames: 'hostName facility hosts'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Logging-Adapters'!

!LogSyslogSender commentStamp: 'gk 11/21/2007 01:31' prior: 0!
An emitter that emits to a remote host (or several) using the syslog protocol in UDP.
The facility is set by number, default is local0, probably just fine.

You may want to set the hostName to something different than localhost.

- Make sure your syslog host (default is localhost) is enabled to listen on port 514.

Test this by:

	SLLog addSyslog.  "Adds a syslog sender emitter to localhost by default"
	SLLog info: 'Testing syslog'!

!LogSyslogSender class methodsFor: 'as yet unclassified' stamp: 'kph 12/1/2008 05:07'!
localhost

	^ self new addHost: 'localhost'! !

!LogSyslogSender methodsFor: 'accessing' stamp: 'gk 11/21/2007 01:10'!
addHost: aString
	"Add a host to send to, 514 is default syslog port."

	self addHost: aString port: 514! !

!LogSyslogSender methodsFor: 'accessing' stamp: 'gk 11/21/2007 01:10'!
addHost: aString port: aPortNumber
	"Add a host name with portnumber."

	hosts add: aString -> (aPortNumber ifNil: [514])! !

!LogSyslogSender methodsFor: 'accessing' stamp: 'gk 11/21/2007 01:07'!
facility: anInteger
	"We keep it multiplied with 8 so we don't need to do that every time we emit."

	facility := anInteger * 8! !

!LogSyslogSender methodsFor: 'accessing' stamp: 'mtf 4/25/2011 01:12'!
filter
	^ nil! !

!LogSyslogSender methodsFor: 'accessing' stamp: 'mtf 4/25/2011 01:12'!
filter: aValuable! !

!LogSyslogSender methodsFor: 'accessing' stamp: 'gk 11/21/2007 00:03'!
hostName
	"Apparently this is not that easy to find cross platform."

	^hostName ifNil: ['localhost']! !

!LogSyslogSender methodsFor: 'accessing' stamp: 'gk 11/21/2007 01:08'!
hostName: aHostName
	"Apparently this is not that easy to find cross platform."

	hostName := aHostName! !

!LogSyslogSender methodsFor: 'initialize-release' stamp: 'kph 12/1/2008 05:06'!
initialize
	self facility: 16. "Use 1 (=user level) or 16-23 (=local use local0-7)"
! !

!LogSyslogSender methodsFor: 'private' stamp: 'mtf 4/29/2011 16:38'!
logEntry: aLogEntry formatted: aLogFormatter
	"Produce a standard message and send it using UDP to all registered hosts. ignores the formatter for now"

	| message d dd |
		d := aLogEntry stamp dayOfMonth.
		dd := d < 10 ifTrue: [' ', d asString] ifFalse: [d asString].
		message := String new writeStream
			 << $< << (facility + aLogEntry numericLevel) asString << $> << aLogEntry stamp monthAbbreviation << ' ' << dd << ' '.
		 
		aLogEntry stamp asTime print24: true on: message.
		message << ' ' << self hostName << '[' << aLogEntry pid << '] ' << aLogEntry level << '(' << aLogEntry numericLevel << '): '.
		aLogEntry category ifNotNil: [ message <<  aLogEntry category asUppercase << ' ' ].
		message << aLogEntry line.
		aLogEntry sender ifNotNil: [ message << ' (' << aLogEntry sender asString << ')' ].

 		hosts do: [:host | self send: message to: host key port: host value]
 ! !

!LogSyslogSender methodsFor: 'private' stamp: 'gk 11/21/2007 00:04'!
send: message to: host port: port
	"Send formatted UDP message to host and port."

	| sock |
	sock := Socket newUDP.
	sock setPeer: NetNameResolver localHostAddress port: port.
	sock sendData: message.
	sock waitForSendDoneFor: Socket standardDeadline.
	sock close! !

!LogSyslogSender methodsFor: 'private' stamp: 'mtf 4/29/2011 16:33'!
wouldLogEntry: aLogEntry
	^ true! !

Object subclass: #LogTestAdapter
	instanceVariableNames: 'output'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Logging-Tests'!

!LogTestAdapter methodsFor: 'as yet unclassified' stamp: 'kph 11/27/2008 01:24'!
defaultBufferSize

	^ 400! !

!LogTestAdapter methodsFor: 'as yet unclassified' stamp: 'kph 11/29/2008 03:58'!
initialize

	output := (String new: self defaultBufferSize) writeStream! !

!LogTestAdapter methodsFor: 'as yet unclassified' stamp: 'mtf 4/29/2011 16:38'!
logEntry: aLogEntry formatted: aLogFormatter

	output << aLogEntry level asUppercase << ' ' << aLogEntry category asUppercase << ' ' << aLogEntry line; cr.

	
 ! !

!LogTestAdapter methodsFor: 'as yet unclassified' stamp: 'kph 11/26/2008 06:46'!
output

	^ output contents! !

!LogTestAdapter methodsFor: 'as yet unclassified' stamp: 'kph 12/3/2008 20:08'!
privateMethodFor: test

	test log privateViolation
 ! !

!LogTestAdapter methodsFor: 'as yet unclassified' stamp: 'mtf 4/29/2011 16:33'!
wouldLogEntry: aLogEntry
	^ true! !

Object subclass: #LogToothpick
	instanceVariableNames: 'category'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Logging-Adapters'!

!LogToothpick commentStamp: 'kph 10/8/2007 06:20' prior: 0!
To use Toothpick logging.

Toothpick event basics:

category - any symbol
level - one of #(#all #debug #info #notice #warn #error #crit #alert #emerg #panic #fatal #off #none)

Messages are not size limited in toothpick, so.. #cr is rendered inline.
However since we reuse our buffer, we consider > bufferMax (10K) to be exceptional, and reinitialize it
if this is exceeded.

All of these levels are natively supported by the toothpick adaptor. 
Other adaptors will accept these levels on their own terms!!

usage:

self log info database query: sqlQuery.
self log database info query: sqlQuery.

Where database is interpreted as a category, and query: is used as an annotation.





!

!LogToothpick class methodsFor: 'as yet unclassified' stamp: 'kph 12/12/2007 17:46'!
LoggingEvent: block

	^ Smalltalk at: #LoggingEvent ifPresent: block  ! !

!LogToothpick class methodsFor: 'as yet unclassified' stamp: 'kph 12/12/2007 13:31'!
LoggingMultiplexer

	^ Smalltalk at: #LoggingMultiplexer  ! !

!LogToothpick class methodsFor: 'as yet unclassified' stamp: 'kph 12/12/2007 13:32'!
LoggingPolicy

	^ Smalltalk at: #LoggingPolicy  ! !

!LogToothpick class methodsFor: 'as yet unclassified' stamp: 'kph 12/12/2007 13:32'!
TimestampLoggingFormatter

	^ Smalltalk at: #TimestampLoggingFormatter  ! !

!LogToothpick class methodsFor: 'as yet unclassified' stamp: 'kph 12/12/2007 13:31'!
TranscriptLogger

	^ Smalltalk at: #TranscriptLogger  ! !

!LogToothpick class methodsFor: 'as yet unclassified' stamp: 'kph 12/12/2007 14:14'!
isLoaded

 Smalltalk at: #LoggingMultiplexer ifAbsent: [ ^false ].
	
 ^ true! !

!LogToothpick class methodsFor: 'as yet unclassified' stamp: 'kph 12/12/2007 14:13'!
reset
	self stop
	self isLoaded ifFalse: [ self error: 'Toothpick is not loaded' ].
	
	self LoggingMultiplexer current stopAllLoggers.
	self LoggingMultiplexer reset. ! !

!LogToothpick class methodsFor: 'as yet unclassified' stamp: 'kph 12/12/2007 14:33'!
startFile: filename
 
| logger1  |
 
	self start.

	self isLoaded ifFalse: [ self error: 'Toothpick is not loaded' ].

	logger1 := FileLogger new
	fileName: filename;
	format: self TimestampLoggingFormatter iso8601Format;
	policy: (self LoggingPolicy new isInterestingBlock: [ :evt | true ]).

	(self LoggingMultiplexer current)
	addLogger: logger1;
 
	startAllLoggers.   
 
 ! !

!LogToothpick class methodsFor: 'as yet unclassified' stamp: 'kph 12/12/2007 14:33'!
startSafeTranscript 
	"
	also we use the threadsafe transcript if available (from the Nile package)"

	| logger1  |

	self start.

	self isLoaded ifFalse: [ self error: 'Toothpick is not loaded' ].


	logger1 := self TranscriptLogger new.

	Smalltalk at: #ThreadSafeTranscript ifPresent: [ :t | 
			Smalltalk at: #Transcript put: t new.
			logger1 := ThreadSafeTranscriptLogger new.
	].
	
	Transcript open.
	 
	logger1 
		name: 'tlog';
		format: self TimestampLoggingFormatter iso8601Format;
		policy: (self LoggingPolicy new isInterestingBlock: [ :evt | true ]).

	(self LoggingMultiplexer current)
		addLogger: logger1;
 		startAllLoggers. ! !

!LogToothpick class methodsFor: 'as yet unclassified' stamp: 'kph 12/12/2007 14:33'!
startTranscript 
 	| logger1  |

	self start.

	self isLoaded ifFalse: [ self error: 'Toothpick is not loaded' ].

	logger1 := self TranscriptLogger new.
		 
	logger1 
		name: 'tlog';
		format: self TimestampLoggingFormatter iso8601Format;
		policy: (self LoggingPolicy new isInterestingBlock: [ :evt | true ]).

	(self LoggingMultiplexer current)
		addLogger: logger1;
 		startAllLoggers. ! !

!LogToothpick methodsFor: 'levels' stamp: 'kph 10/8/2007 05:46'!
alert
	
	 level := #alert! !

!LogToothpick methodsFor: 'levels' stamp: 'kph 10/8/2007 05:44'!
all
	
	level := #all! !

!LogToothpick methodsFor: 'accessing' stamp: 'kph 8/7/2007 23:25'!
category
	"Answer the value of category"

	^ category! !

!LogToothpick methodsFor: 'accessing' stamp: 'kph 8/7/2007 23:25'!
category: anObject
	"Set the value of category"

	category := anObject! !

!LogToothpick methodsFor: 'levels' stamp: 'kph 10/8/2007 05:44'!
crit
	
	level := #crit! !

!LogToothpick methodsFor: 'levels' stamp: 'kph 10/8/2007 05:44'!
critical
	
	level := #crit! !

!LogToothpick methodsFor: 'levels' stamp: 'kph 10/8/2007 05:45'!
debug
	
	level := #debug! !

!LogToothpick methodsFor: 'accessing' stamp: 'kph 12/12/2007 15:21'!
dynamic2: aSymbol
	"Uknown unary selectors are used after the category"

	self << aSymbol << ' '.! !

!LogToothpick methodsFor: 'levels' stamp: 'kph 10/8/2007 05:45'!
emerg
	
	level := #emerg! !

!LogToothpick methodsFor: 'levels' stamp: 'kph 10/8/2007 05:46'!
emergency
	
	level := #emerg! !

!LogToothpick methodsFor: 'as yet unclassified' stamp: 'kph 11/28/2008 04:08'!
endEntry
	
	isIgnore ifFalse: [ 

		self class LoggingEvent: [ :event | event category: category level: level message: stream contents ].
	].

	stream reset.! !

!LogToothpick methodsFor: 'levels' stamp: 'kph 10/8/2007 05:40'!
error

	level := #error! !

!LogToothpick methodsFor: 'levels' stamp: 'kph 10/8/2007 05:42'!
fatal

	level := #fatal! !

!LogToothpick methodsFor: 'accessing' stamp: 'mtf 4/25/2011 01:13'!
filter
	^ nil! !

!LogToothpick methodsFor: 'accessing' stamp: 'mtf 4/25/2011 01:13'!
filter: aValueable! !

!LogToothpick methodsFor: 'levels' stamp: 'kph 10/8/2007 05:40'!
info

	level := #info! !

!LogToothpick methodsFor: 'levels' stamp: 'mtf 4/29/2011 16:38'!
logEntry: aLogEntry formatted: aLogFormatter
! !

!LogToothpick methodsFor: 'levels' stamp: 'kph 10/8/2007 05:47'!
notice

	level := #notice ! !

!LogToothpick methodsFor: 'levels' stamp: 'kph 10/8/2007 05:43'!
off
	level := #off! !

!LogToothpick methodsFor: 'levels' stamp: 'kph 10/8/2007 05:40'!
panic

	level := #panic! !

!LogToothpick methodsFor: 'levels' stamp: 'kph 12/12/2007 16:30'!
temp
	
	level := #debug! !

!LogToothpick methodsFor: 'levels' stamp: 'kph 10/8/2007 05:40'!
warn

	level := #warn! !

!LogToothpick methodsFor: 'levels' stamp: 'mtf 4/29/2011 16:33'!
wouldLogEntry: aLogEntry
	^ false! !

Object subclass: #LogWatchdog
	instanceVariableNames: 'buffer collectMinutes fromEmail smtpAccount smtpPassword smtpServer threshold timer toEmail'
	classVariableNames: 'Buffer CollectMinutes Default FromEmail SMTPAccount SMTPPassword SMTPServer Threshold Timer ToEmail'
	poolDictionaries: ''
	category: 'Logging-Adapters'!

!LogWatchdog commentStamp: 'gk 6/1/2007 13:10' prior: 0!
This log emitter has a threshold. If log messages comes in that are higher than the threshold we send an email to an address with the log message. We then set a timer and any messages coming in over the threshold until the timer has run out we bundle together in another email, send it and set the timer again.

If the timer runs out and there are no buffered messages we are done. This approach prevents drowning in emails.

Use like this:

SLLogWatchDog
	fromEmail: 'mysystem@brrrrrrrrrrr.org';
	toEmail: 'me@brrrrrrrrrrr.org';
	start

Stop:

SLLogWatchDog stop!

!LogWatchdog class methodsFor: 'configuration' stamp: 'kph 12/1/2008 05:25'!
SMTPServer: aServerName
	self default smtpServer: aServerName! !

!LogWatchdog class methodsFor: 'configuration' stamp: 'kph 12/1/2008 05:25'!
account: anEmail
	self default account: anEmail! !

!LogWatchdog class methodsFor: 'configuration' stamp: 'kph 12/1/2008 05:26'!
collectMinutes: minutes
	self default collectMinutes: minutes! !

!LogWatchdog class methodsFor: 'configuration' stamp: 'kph 12/1/2008 05:25'!
default 

	^ Default ifNil: [ Default := self new ]! !

!LogWatchdog class methodsFor: 'configuration' stamp: 'kph 12/1/2008 05:26'!
fromEmail: anEmailAddress
	self default fromEmail: anEmailAddress! !

!LogWatchdog class methodsFor: 'configuration' stamp: 'kph 12/1/2008 05:27'!
password: aString
	self default password: aString! !

!LogWatchdog class methodsFor: 'configuration' stamp: 'kph 12/1/2008 05:28'!
toEmail: anEmailAddress
	self default toEmail: anEmailAddress! !

!LogWatchdog methodsFor: 'as yet unclassified' stamp: 'kph 12/1/2008 05:19'!
account: anEmail
	smtpAccount := anEmail  ! !

!LogWatchdog methodsFor: 'as yet unclassified' stamp: 'kph 12/1/2008 05:19'!
collectMinutes: minutes
	collectMinutes := minutes ! !

!LogWatchdog methodsFor: 'as yet unclassified' stamp: 'kph 2/24/2009 16:37'!
critical: aBlock

		^ LogCurrent critical: aBlock! !

!LogWatchdog methodsFor: 'as yet unclassified' stamp: 'mtf 4/25/2011 01:14'!
filter
	^ nil! !

!LogWatchdog methodsFor: 'as yet unclassified' stamp: 'mtf 4/25/2011 01:14'!
filter: aValuable! !

!LogWatchdog methodsFor: 'as yet unclassified' stamp: 'kph 12/1/2008 05:20'!
fromEmail: anEmailAddress
	fromEmail := anEmailAddress! !

!LogWatchdog methodsFor: 'as yet unclassified' stamp: 'kph 12/1/2008 05:20'!
initialize
	"Default threshold ignores debug and info but includes warnings and worse."

	"self initialize"

	threshold := 4.
	buffer := Timer := nil.
	collectMinutes := 300.
	smtpServer := 'localhost'! !

!LogWatchdog methodsFor: 'as yet unclassified' stamp: 'mtf 4/29/2011 16:38'!
logEntry: aLogEntry formatted: aLogFormatter

	"why is SLLogWatchdog written class side?"

	"This method looks messy, and it is - but it is written to be as fast as possible."

	"Is this message over the threshold?"
	| message now |
	
	aLogEntry numericLevel <= Threshold
		ifTrue: [
			message := String new writeStream.
			message << aLogEntry stamp printString << ' [' << aLogEntry pid << '] - ' << aLogEntry level << '(', aLogEntry numericLevel << '): '.
			aLogEntry category ifNotNil: [ message << aLogEntry category asUppercase << ' ' ].
			message << aLogEntry line << ' (' << aLogEntry sender asString << ')'.
			Buffer
				ifNotNil: [
					"Add message to the buffer and possibly send it if the timer is out."
					self critical: [ 
						Buffer nextPutAll: message contents.
						now := DateAndTime now asSeconds.
						"Is the timer out? If so, send the buffer."
					 	now > Timer ifTrue: [ self sendBuffer]
					]
				]
				ifNil: [
					"No buffer to send nor add message to. Set the timer and send the message."
					Timer := DateAndTime now asSeconds + CollectMinutes.
					Buffer := String new writeStream.
					self sendMessage: message contents]]
		ifFalse: [
			"Ok, no message this time - do we have a buffer to send when the timer goes out?"
			Buffer ifNotNil: [
				now := DateAndTime now asSeconds.
				"Is the timer out? If so, if the buffer has content - send the buffer and reset Timer. Otherwise nil Buffer."
				now > Timer ifTrue: [
					Buffer isEmpty
						ifTrue: [Buffer := nil]
						ifFalse: [self sendBuffer]]]]! !

!LogWatchdog methodsFor: 'as yet unclassified' stamp: 'kph 2/24/2009 16:27'!
mailFrom: sender to: receivers text: headerAndBody

	| smtpClient |
	smtpClient := SMTPClient new.
	smtpAccount ifNotNil: [
		smtpClient user: smtpAccount; password: smtpPassword.

		[smtpClient openOnHost: (NetNameResolver addressForName: smtpServer timeout: 20) port: 25.
		[smtpClient mailFrom: sender to: receivers text: headerAndBody]
				ensure: [smtpClient close]]
				on: Error do: [:ex |
					self log error: 'Error when sending email: ', ex asString]
	]! !

!LogWatchdog methodsFor: 'as yet unclassified' stamp: 'kph 12/1/2008 05:23'!
password: aString
	smtpPassword := aString! !

!LogWatchdog methodsFor: 'as yet unclassified' stamp: 'kph 12/1/2008 05:23'!
reInitialize

	buffer := timer := nil.
	self start! !

!LogWatchdog methodsFor: 'as yet unclassified' stamp: 'kph 12/1/2008 05:24'!
sendBuffer
	"Send the buffer, reset timer, empty buffer."

	self sendMessage: buffer contents.
	timer := DateAndTime now asSeconds + 60.
	buffer := String new writeStream! !

!LogWatchdog methodsFor: 'as yet unclassified' stamp: 'kph 12/1/2008 05:24'!
sendMessage: msg
	"Send the message."

"Transcript show: 'Watchdog sending: ', msg;cr."
	toEmail isEmptyOrNil
		ifFalse: [
			self
				mailFrom: FromEmail
				to: (Array with: ToEmail)
				text: 'Subject: Important log messages

Log messages above threshold:
', msg]
! !

!LogWatchdog methodsFor: 'as yet unclassified' stamp: 'kph 12/1/2008 05:19'!
smtpServer: aServerName
	smtpServer := aServerName ! !

!LogWatchdog methodsFor: 'as yet unclassified' stamp: 'mtf 4/29/2011 16:33'!
wouldLogEntry: aLogEntry
	^ aLogEntry numericLevel <= Threshold! !

!Object methodsFor: '*logging-core' stamp: 'kph 12/2/2008 02:45'!
log
	"This method provides the univeral entry point for all logging mechanisms"
	 
	^ LogCurrent value
		inSender: thisContext sender 
	
  
	 ! !

!Object methodsFor: '*logging-core' stamp: 'mtf 4/3/2009 23:48'!
log: aString
"A convenience method for logging one string at temp priority"

	self log temp: aString! !

!Object methodsFor: '*logging-core' stamp: 'kph 11/27/2008 01:48'!
logDetailedOn: stream
	self hasContentsInExplorer ifFalse: [ ^ self printOn: stream ].
 
	(self respondsTo: #explorerContents)
		ifTrue: [ 
			self explorerContents do: [ :each | stream << each asString; cr ].
			^ self
		].
	"For all others, show named vars first, then indexed vars"
	self class allInstVarNames keysAndValuesDo: [:index :each |
		stream << each << ': ' << (self instVarAt: index) printString ; << ' '.
	].
	(1 to: self basicSize) do: [:index |
		stream << index << ': ' << (self basicAt: index) printString << ' '.
	].
		 ! !

!Object methodsFor: '*logging-core' stamp: 'kph 11/20/2007 09:58'!
logDumperOn: stream

	self storeOn: stream! !

!Object methodsFor: '*logging-core' stamp: 'kph 10/8/2007 08:16'!
logOn: stream

	self printOn: stream! !

!MethodContext methodsFor: '*logging' stamp: 'mtf 1/27/2008 00:48'!
asMessage
	| selector |
	selector := self selector.
	^ selector isUnary
		ifTrue: [Message selector: selector]
		ifFalse: [Message selector: selector arguments:
		((1 to: selector keywords size) collect: [:i | self at: i])]! !

Exception subclass: #LogException
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Logging-Core'!

!LogException commentStamp: 'gg 2/24/2001 22:18' prior: 0!
This class is used for tracing a call to the log.
Using
	oc:= LogException signal.
you got a ordered collection with a line per stack description.!

!LogException methodsFor: 'exceptionDescription' stamp: 'test 11/28/2008 02:22'!
defaultAction
	 
	^self getLine.
	
 ! !

!LogException methodsFor: 'accessing' stamp: 'gg 2/24/2001 21:34'!
getLine
	| ctx oc |
	oc  := OrderedCollection new..
	ctx := self privGetContext.

	"Note: The following is an open-coded version of ContextPart>>stackOfSize: 
	since this method may be called during a low space condition and we  
	might run out of space for allocating the full stack."
	[ctx == nil]
		whileFalse: 
			[oc add:  (ctx asString).
			ctx := ctx sender].
	^ oc! !

!LogException methodsFor: 'private' stamp: 'test 11/28/2008 02:22'!
privGetContext
	^ self signalerContext.! !

!DateAndTime methodsFor: '*logging-core' stamp: 'kph 2/24/2009 16:05'!
printLocalOn: aStream
	"Print as per ISO 8601 sections 5.3.3 and 5.4.1.
	
	ignore the timezone offset
	
	If printLeadingSpaceToo is false, prints either:
		'YYYY-MM-DDThh:mm:ss.s' (for positive years) or '-YYYY-MM-DDThh:mm:ss.s' (for negative years)
	If printLeadingSpaceToo is true, prints either:
		' YYYY-MM-DDThh:mm:ss.s' (for positive years) or '-YYYY-MM-DDThh:mm:ss.s' (for negative years)
	"

	self printYMDOn: aStream withLeadingSpace: false.
	aStream nextPut: $T.
	self printHMSOn: aStream.
	self nanoSecond ~= 0 ifTrue:
		[ | z ps |
		ps := self nanoSecond printString padded: #left to: 9 with: $0.
		z := ps findLast: [ :c | c asciiValue > $0 asciiValue ].
		(z > 0) ifTrue: [aStream nextPut: $.].
		ps from: 1 to: z do: [ :c | aStream nextPut: c ] ].
! !

ProcessLocalVariable subclass: #LogCurrent
	instanceVariableNames: 'pid properties sender stream level category adapter formatter source'
	classVariableNames: 'Levels Protect'
	poolDictionaries: ''
	category: 'Logging-Core'!

LogCurrent class
	instanceVariableNames: 'default'!

!LogCurrent commentStamp: 'kph 12/2/2008 06:06' prior: 0!
Architecture:
================

LogCurrent the class is also a ProcessSpecific variable that returns an different instance of LogCurrent for each process. Each of these maintains a buffer for that process's log output.

Users use 'self log' to obtain a LogCurrent buffer instance for the current process from 'LogCurrent value'. Since LogCurrent buffers are per-process they are threadsafe, and can maintain state between calls (self log properties).

Each LogCurrent instance maintains a stream (on its buffer) and provides the users front end interface for convenient logging.

Each LogCurrent instance maintains an adapter, when a log line is completed (#endEntry), the line is sent to the adapter, the adapter being the backend interface.

The adapter may be any class that implements  #logTime: stamp line: line level: level category: category sender: sender pid: pid
e.g. Transcript.

The adapter may also be a LogAdapterSet, in which case, the line is sent to each adapter contained in the set in turn. LogAdapterSets may be used to allow a number of processes to share the same set of adapters. LogAdapterSets may be arbitrarily nested, and may define a filterBlock. e.g. self filterBlock: [ :level :category |  #(#error #debug #info) includes: level ]. AdapterSet may be subclassed if more specific filtering behaviour is required.

Any changes to LogAdapterSets anywhere are protected by a single Semaphore, but use of adapters for output is protected instead by using a safer implementation of #do: to iterate over the set should an adapter be removed by a different thread.

All processes that do not have a LogCurrent instance defined will copy the default LogCurrent instance, and will therefore share the same adapter/adapterset as the default.

When you assign a bespoke  LogCurrent instance to a specific process, it is up to you to set its adapter. It can include the global adapter/adapterset, nested within the new set you give it.

Additional logging utilities and tools may be defined by subclassing LogCurrent. LogAdapterSet may be subclassed to provide more interesting filtering behaviour. And individual Adapters may also be configured or subclassed as desired.

Adapters:
===========

1) LogHistory e.g. LogHistory size: 20.
If the LogHistory adapter is present in a LogAdapterSet, then any new adapters added to that set have the recorded history sent to them immediately. Only one LogHistory adapter per set is allowed.

2) Blocks, LogAdapterSet may contain a block which returns a real adaper or nil. This enables adapters to be written to if enabled, or ignored if not available (see SLLogMorph below).

3) Transcript - the standard Transcript is used, and the line is output in one hit within the Transcripts own protection semaphore.  

4) LogEngine e.g. LogEngine for: 'seaside'.
This adapter is for the LogEngine backend.

5) SLLog
This adapter is for the SimpleLog backend.

6) Toothpick
This adapter is for the Toothpick backend. (Not yet supported)

Direct to 'SimpleLog' Components
7) SLSyslogSender e,g, SLSyslogSender new addHost: 'log.somewhere.else'.
or SLSyslogSender localhost.

8) SLLogWatchdog e.g. SLLogWatchdog default.
Emails log entries over the given threshold

9) SLLogMorph e.g. [ SLLogMorph current ].
The block will be evaluated for every log entry. Therefore this construct
will only write to LogMorph if it is open.

10) SLLogFile named: 'current.log'.
Subclass and override #output*, or add #output* methods for additional file output formats.
Or use (SLLogFile named: 'custom.log') output: [ :entry | entry stamp, '-', entry line ]

Configuration:

To change the global, or local LogCurrent instance

LogCurrent default: MyLogBuffer new
LogCurrent value: MyLogBuffer new

To change the global adapters:

LogCurrent default adapter: (LogAdapterSet with: Transcript).
LogCurrent default adapter add: (LogHistory size: 20).

To change the local adapter, logging to the current global adapters, and the local transcript, with a custom policy.

newLog := MyOwnLogCurrent new.
newLog adapter: (LogAdapterSet with: (LogCurrent default adapter) with: Transcript).
LogCurrent value: newLog.

Usable LogCurrent instance is returned from <code>self log<code> in whatever context it is used. The buffer is instanciated on a per process basis, therefore I am threadsafe already, and can reuse my buffers and maintain state as we progress through the code which we are logging. Unusually for a logging framework this allows us to perform timing tasks for our clients. If the code forks then a new buffer instance will handle the new forked process.

For timing use:

= self log timeTotal: #eventA.
= self log timeDelta: #eventB.

Convenient usage: 'annotated values'

This uses the logging level defined by LogRouter-temp, and is considered a temporary log message for debugging purposes. For temporary log messages the specific level is rarely important. There is by default a global preference for disabling temporary log level messages. Of course your subclasses can override this behaviour.

= self log x: 10 y: 20 z: 30.

To select a specific log severity level (i.e. not the temporary one featured above).

= self log info x: 10 y: 20 z: 30.

To log the current method's input parameters.

Rectangle-#corner: topLeftPoint extent: heightWidthPoint

= self log debug this.

would be the same as

= self log corner: topLeftPoint extent: heightWidthPoint.

To log the current methods variables:

= self log debug it instance.
= self log debug it temps.
= self log debug it classVar.
= self class log debug it instance.
= self log debug vars all.

Levels of detail:

When logging it is useful to be able to specify easily the level of detail that we desire from the items we are given.
Here we define 4 levels of detail.

1. Annotated values: the data points are simple values intended to be displayed on one line.
the method selector is used to annotate the vaues for readability.

= self log info r: 20 theta: 0.5.

2. Verbatim: Stream to the Log

= (self log info << 'User Directory:' << aFilePath) write.

3. Detailed: human readable presentation: A detailed presentation of the objects internal state intended for human readability.
    The detail printed is based upon the object's explorer contents, as defined in #logDetailedOn:

= self log info detailed: myDictionary.

4. Data Dumper: A machine readable serialized text format.
    The format printed is defined in #logDumperOn:

    For those perl coders out there who really like being able to recreate their data from logs!!

= self log info dumper: myTransaction.
!

LogCurrent class
	instanceVariableNames: 'default'!

!LogCurrent class methodsFor: 'as yet unclassified' stamp: 'kph 12/2/2008 00:35'!
cleanUp
	super cleanUp.
	self initialize.
	
 ! !

!LogCurrent class methodsFor: 'as yet unclassified' stamp: 'kph 12/2/2008 02:25'!
critical: aBlock

	^ Protect critical: aBlock! !

!LogCurrent class methodsFor: 'instanciation' stamp: 'kph 12/2/2008 00:32'!
debugMode
	"enable some transcript printing of DNU's etc"
	^ false! !

!LogCurrent class methodsFor: 'as yet unclassified' stamp: 'kph 3/5/2009 22:36'!
default
	 
	default ifNil: [ (default := self new) adapters addHistoryLog ]. 
	
	^ default! !

!LogCurrent class methodsFor: 'as yet unclassified' stamp: 'kph 12/9/2008 21:46'!
default: router
  
	default ifNotNil: [
		
		(router isNil or: [router isKindOf: self]) ifFalse: [ ^ self error: 'use nil for default, or must use an instance of LogCurrent (or subclasses)'. ].
				
		default processesWithMyAdapterDo: [ :proc | 
		
			proc environmentAt: self put: router.
	
		].
	].

	default := router! !

!LogCurrent class methodsFor: 'as yet unclassified' stamp: 'kph 3/13/2010 12:12'!
initialize
 	
	default := nil.
	Protect := Semaphore forMutualExclusion.
	"Protect := Monitor new."
	
	Preferences addPreference: #enableTemporaryDebugMessages
		categories: #('logging') default: true 
		balloonHelp: 'Enable logging of temporary debug messages'.
		
	"Preferences addPreference: #logDebug
		categories: #('logging') default: true 
		balloonHelp: 'Enable logging of debug messages'."
	
	Levels := IdentityDictionary new.

	"Same as Toothpick list"
	#(#none #off #fatal #panic #emergency #alert #critical #error #warn #notice #info #debug #temp #all) 

	keysAndValuesDo: [ :n :l | Levels at: l put: n ].! !

!LogCurrent class methodsFor: 'as yet unclassified' stamp: 'kph 12/2/2008 00:42'!
key

	^ #Log! !

!LogCurrent class methodsFor: 'as yet unclassified' stamp: 'kph 12/2/2008 00:37'!
levelIndexAt: level

	 ^ Levels at: level ifAbsent: [ 9 ]! !

!LogCurrent class methodsFor: 'as yet unclassified' stamp: 'kph 2/24/2009 02:01'!
newHistoryLog
	 
	^ LogCurrent new adapter: (LogAdaptersSet with: (LogHistory size: 30)); yourself ! !

!LogCurrent class methodsFor: 'as yet unclassified' stamp: 'mtf 5/4/2011 16:25'!
value
 
	"Answer the current value for this variable in the current context."
	
	"if a fork has occured the returned value will be from the previous process, we copy it 
	and make the duplicate our own"
	
	| log |
	
	^ Processor activeProcess in: [ :p | 
		
		log := p environmentAt: self key ifAbsent: [ self default ].
		log pid = p hash ifFalse: [
			log := log copy.
			log setPid: p hash.
			self value: log].
		
		log

 	]! !

!LogCurrent methodsFor: 'public' stamp: 'mtf 4/4/2009 01:00'!
<< anObject
"Write an object to the log using its printOn: method"

	self isIgnore ifFalse: [ 
		anObject isString 
			ifTrue: [ stream nextPutAll: anObject ]
			ifFalse: [ anObject logOn: stream ]
	].! !

!LogCurrent methodsFor: 'public' stamp: 'mtf 4/4/2009 01:02'!
<<< anObject
"Write this object and its contents to the log using the ObjectExplorer"

	^ self detailed: anObject! !

!LogCurrent methodsFor: 'accessing' stamp: 'kph 12/1/2008 04:03'!
adapter

	^ adapter! !

!LogCurrent methodsFor: 'accessing' stamp: 'kph 12/1/2008 04:03'!
adapter: a

	adapter := a! !

!LogCurrent methodsFor: 'accessing' stamp: 'kph 2/24/2009 02:04'!
adapters

	adapter ifNil: [ adapter := LogAdaptersSet new ]. 

	(adapter isKindOf: LogAdaptersSet) ifTrue: [ ^ adapter ].
	
	^adapter := LogAdaptersSet with: adapter.
	
	! !

!LogCurrent methodsFor: 'levels' stamp: 'mtf 4/3/2009 23:55'!
alert
"High priority: 4 of 11:
fatal panic emergency ALERT critical error warn notice info debug temp"

	level := #alert! !

!LogCurrent methodsFor: 'levels' stamp: 'mtf 4/3/2009 23:55'!
alert: aString
"Logs aString at the alert level. Alert is level 4 of 11:
fatal panic emergency ALERT critical error warn notice info debug temp"

	self alert << aString ; endEntry! !

!LogCurrent methodsFor: 'line-buffer' stamp: 'kph 12/1/2008 22:19'!
beginEntry
	"buffers grow automatically but if they get beyond bufferMax we assume that there
	 was an unusually large message and we dont want to keep our buffer at this huge size forever
	so we reset it"
	
	stream position > self bufferMax ifTrue: [ self initializeUnlimitedBuffer ].	 
	stream reset. 
 
	self temp.! !

!LogCurrent methodsFor: 'initialize-release' stamp: 'kph 12/1/2008 22:19'!
bufferMax
	
	^ 2048! !

!LogCurrent methodsFor: 'line-buffer' stamp: 'mtf 5/4/2011 16:24'!
buildEntry
"The full message to be logged"

	^ self buildEntryHead
		stamp: DateAndTime now;
		line: stream contents;
		processDescription: Processor activeProcess name;
		yourself! !

!LogCurrent methodsFor: 'line-buffer' stamp: 'mtf 4/29/2011 16:40'!
buildEntryHead
"The metadata about the message to be logged, mostly for pre-filtering"

	^ LogEntry new
		level: level;
		category: self category;
		sender: sender;
		pid: self pid;
		yourself! !

!LogCurrent methodsFor: 'accessing' stamp: 'mtf 4/24/2011 19:36'!
category
	category ifNotNil: [^ category].
	sender ifNil: [^ nil].
	^ sender receiver class theNonMetaClass name! !

!LogCurrent methodsFor: 'line-buffer' stamp: 'kph 11/29/2008 23:44'!
cr

	self isIgnore ifFalse: [ stream cr ].! !

!LogCurrent methodsFor: 'levels' stamp: 'mtf 4/3/2009 23:34'!
critical
"High priority: 5 of 11:
fatal panic emergency alert CRITICAL error warn notice info debug temp"

	level := #critical! !

!LogCurrent methodsFor: 'levels' stamp: 'mtf 4/3/2009 23:57'!
critical: aString
"Logs aString at the critical level. Critical is level 5 of 11:
fatal panic emergency alert CRITICAL error warn notice info debug temp"

	self critical << aString ; endEntry! !

!LogCurrent methodsFor: 'levels' stamp: 'mtf 4/3/2009 23:35'!
debug
"Low priority: 10 of 11:
fatal panic emergency alert critical error warn notice info DEBUG temp"

	level := #debug! !

!LogCurrent methodsFor: 'levels' stamp: 'mtf 4/3/2009 23:57'!
debug: aString
"Logs aString at the debug level. Debug is level 10 of 11:
fatal panic emergency alert critical error warn notice info DEBUG temp"

	self debug << aString ; endEntry! !

!LogCurrent methodsFor: 'initialize-release' stamp: 'kph 11/29/2008 03:04'!
defaultBufferSize

	^ 256! !

!LogCurrent methodsFor: 'initialize-release' stamp: 'kph 11/29/2008 03:04'!
defaultLimitedBufferSize

	self subclassResponsibility! !

!LogCurrent methodsFor: 'public' stamp: 'mtf 4/4/2009 01:02'!
detailed: anObject
"Write this object and its contents to the log using the ObjectExplorer"

	self isIgnore ifFalse: [ 
		anObject isString 
			ifTrue: [ stream nextPutAll: anObject ]
			ifFalse: [ anObject logDetailedOn: stream ].
		self endEntry.
	].! !

!LogCurrent methodsFor: 'private' stamp: 'kph 12/2/2008 00:33'!
doesNotUnderstand: aMessage
	"for debugging"
	self class debugMode ifTrue: [ Transcript show: self class name, ' DNU ', aMessage printString; cr ].
	
	aMessage selector isUnary
		ifTrue: [ category := aMessage selector ]
		ifFalse: [ self selectorAndArgs: aMessage]
 ! !

!LogCurrent methodsFor: 'public' stamp: 'mtf 4/4/2009 00:41'!
dumper: anObject
"Log anObject using its custom log dumper method"
	 
	self isIgnore ifFalse: [ 
		anObject logDumperOn: stream 
		self endEntry.		
	]! !

!LogCurrent methodsFor: 'levels' stamp: 'mtf 4/3/2009 23:35'!
emergency
"Top priority: 3 of 11:
fatal panic EMERGENCY alert critical error warn notice info debug temp"

	level := #emergency! !

!LogCurrent methodsFor: 'levels' stamp: 'mtf 4/3/2009 23:57'!
emergency: aString
"Logs aString at the emergency level. Emergency is level 3 of 11:
fatal panic EMERGENCY alert critical error warn notice info debug temp"

	self emergency << aString ; endEntry! !

!LogCurrent methodsFor: 'line-buffer' stamp: 'mtf 1/27/2008 00:43'!
end 

	^ self endEntry! !

!LogCurrent methodsFor: 'line-buffer' stamp: 'mtf 4/29/2011 16:40'!
endEntry

	self isIgnore ifFalse: [self logEntry: self buildEntry].
	sender := nil.
	stream reset.
! !

!LogCurrent methodsFor: 'levels' stamp: 'mtf 4/3/2009 23:35'!
error
"Normal priority: 6 of 11:
fatal panic emergency alert critical ERROR warn notice info debug temp"

	level := #error! !

!LogCurrent methodsFor: 'levels' stamp: 'mtf 4/3/2009 23:50'!
error: aString
"Logs aString at the error level. Error is level 6 of 11:
fatal panic emergency alert critical ERROR warn notice info debug temp"

	self error << aString ; endEntry! !

!LogCurrent methodsFor: 'levels' stamp: 'mtf 4/3/2009 23:36'!
fatal
"Top priority: 1 of 11:
FATAL panic emergency alert critical error warn notice info debug temp"

	level := #fatal! !

!LogCurrent methodsFor: 'levels' stamp: 'mtf 4/3/2009 23:58'!
fatal: aString
"Logs aString at the fatal level. Fatal is level 1 of 11:
FATAL panic emergency alert critical error warn notice info debug temp"

	self fatal << aString ; endEntry! !

!LogCurrent methodsFor: 'accessing' stamp: 'mtf 5/3/2011 10:46'!
formatter
	^ formatter ifNil: [formatter := LogFormatter new]! !

!LogCurrent methodsFor: 'accessing' stamp: 'mtf 4/27/2011 18:58'!
formatter: aFormatter
	formatter := aFormatter! !

!LogCurrent methodsFor: 'public' stamp: 'mtf 4/29/2011 16:56'!
from: anObject
	source := anObject! !

!LogCurrent methodsFor: 'accessing' stamp: 'kph 12/1/2008 03:58'!
identity
	"we get our identity from the adapter, 
	 although there may be many instances of us, all copies will share the same adapter"

	^ adapter! !

!LogCurrent methodsFor: 'public' stamp: 'mtf 5/5/2011 19:35'!
ifActiveDo: aBlock
"Evaluate aBlock only if I would actually produce any output for the set level and category"

	self isIgnore ifTrue: [^ self].
	(adapter wouldLogEntry: self buildEntryHead) ifFalse: [^ self].
	aBlock valueWithPossibleArgument: self.
	stream position = 0 ifFalse: [self endEntry]! !

!LogCurrent methodsFor: 'levels' stamp: 'kph 11/29/2008 23:28'!
ignore

	level := nil! !

!LogCurrent methodsFor: 'accessing' stamp: 'kph 11/29/2008 03:53'!
inSender: aContext
	sender := aContext.
	self beginEntry.! !

!LogCurrent methodsFor: 'levels' stamp: 'mtf 4/3/2009 23:39'!
info
"Normal priority: 9 of 11:
fatal panic emergency alert critical error warn notice INFO debug temp"

	level := #info! !

!LogCurrent methodsFor: 'levels' stamp: 'mtf 4/3/2009 23:58'!
info: aString
"Logs aString at the info level. Info is level 9 of 11:
fatal panic emergency alert critical error warn notice INFO debug temp"

	self info << aString ; endEntry! !

!LogCurrent methodsFor: 'initialize-release' stamp: 'mtf 4/27/2011 18:59'!
initialize

	self formatter: LogFormatter new.
	self initializeUnlimitedBuffer! !

!LogCurrent methodsFor: 'initialize-release' stamp: 'kph 11/29/2008 03:03'!
initializeLimitedBuffer

	stream := (LimitedWriteStream on: (String new: self defaultBufferSize)).
	stream setLimit: self defaultBufferSize limitBlock: nil.! !

!LogCurrent methodsFor: 'initialize-release' stamp: 'kph 11/29/2008 03:03'!
initializeUnlimitedBuffer

	stream := WriteStream on: (String new: self defaultBufferSize).
 ! !

!LogCurrent methodsFor: 'testing' stamp: 'kph 12/1/2008 22:42'!
isGlobal

	^ LogCurrent default adapter == self adapter
! !

!LogCurrent methodsFor: 'testing' stamp: 'kph 12/3/2008 20:11'!
isIgnore 

	^ (level = nil or: [ adapter isNil ]) 
			or: [ adapter isCollection and: [ adapter isEmpty ] ]
 ! !

!LogCurrent methodsFor: 'public' stamp: 'mtf 4/4/2009 00:57'!
it
"Answers a reporter on the calling object. See the class comment in LogReport"

	^ self my! !

!LogCurrent methodsFor: 'accessing' stamp: 'mtf 5/3/2011 18:51'!
level
	^ level! !

!LogCurrent methodsFor: 'accessing' stamp: 'mtf 5/3/2011 18:53'!
level: aSymbol
	level := aSymbol! !

!LogCurrent methodsFor: 'line-buffer' stamp: 'mtf 5/3/2011 10:46'!
logEntry: aLogEntry

	self isIgnore ifTrue: [^ self].
	adapter logEntry: aLogEntry formatted: self formatter! !

!LogCurrent methodsFor: 'public' stamp: 'mtf 5/3/2011 19:02'!
logError: anException
"Log a squeak error. by default, dumps the error to a file and makes a note of the error and the filename with the details"

	self numericLevel: (self numericLevel max: (Levels at: #error)).
	self logEntry: (ErrorLogEntry new
		error: anException;
		level: level;
		category: self category;
		sender: sender;
		pid: self pid;
		stamp: DateAndTime now;
		yourself)! !

!LogCurrent methodsFor: 'public' stamp: 'mtf 4/4/2009 00:57'!
my
"Answers a reporter on the calling object. See the class comment in LogReport"

	^ self report: sender receiver! !

!LogCurrent methodsFor: 'levels' stamp: 'mtf 4/3/2009 23:38'!
notice
"Normal priority: 8 of 11:
fatal panic emergency alert critical error warn NOTICE info debug temp"

	level := #notice! !

!LogCurrent methodsFor: 'levels' stamp: 'mtf 4/3/2009 23:58'!
notice: aString
"Logs aString at the notice level. Notice is level 8 of 11:
fatal panic emergency alert critical error warn NOTICE info debug temp"

	self notice << aString ; endEntry! !

!LogCurrent methodsFor: 'accessing' stamp: 'mtf 5/3/2011 18:52'!
numericLevel
	^ self class levelIndexAt: level! !

!LogCurrent methodsFor: 'accessing' stamp: 'mtf 5/3/2011 18:56'!
numericLevel: anInteger
	level := Levels keyAtValue: anInteger ifAbsent: [^ self]! !

!LogCurrent methodsFor: 'levels' stamp: 'mtf 4/3/2009 23:36'!
panic
"Top priority: 2 of 11:
fatal PANIC emergency alert critical error warn notice info debug temp"

	level := #panic! !

!LogCurrent methodsFor: 'levels' stamp: 'mtf 4/3/2009 23:58'!
panic: aString
"Logs aString at the panic level. Panic is level 2 of 11:
fatal PANIC emergency alert critical error warn notice info debug temp"

	self panic << aString ; endEntry! !

!LogCurrent methodsFor: 'accessing' stamp: 'mtf 4/24/2011 19:39'!
pid
	^ pid ifNil: [Processor activeProcess hash]
		
! !

!LogCurrent methodsFor: 'accessing' stamp: 'kph 12/1/2008 22:55'!
postCopy

	stream := stream copy.! !

!LogCurrent methodsFor: 'public' stamp: 'mtf 4/4/2009 01:09'!
privateViolation
"Log a warning if this method is called from another class. That is, if you have a ''private'' method and want to know if somebody is calling the private method who shouldn't be, use this"

	(sender receiver == sender sender receiver) ifFalse: [self warn << 'PRIVATE VIOLATION: '; this]! !

!LogCurrent methodsFor: 'accessing' stamp: 'kph 12/2/2008 00:54'!
processesWithMyAdapterDo: aBlock

 	"find all the processes with aLogging instance with the same adapter as me"
 	
	Process allSubInstancesDo: [ :ea | 
		(ea environmentAt: self class key) ifNotNilDo: [ :log | 
			log adapter == adapter ifTrue: [ aBlock value: ea ]
		]	
	]! !

!LogCurrent methodsFor: 'public' stamp: 'kph 12/1/2008 02:03'!
properties

	^ properties ifNil: [ properties := IdentityDictionary new ]! !

!LogCurrent methodsFor: 'public' stamp: 'kph 12/12/2007 17:18'!
report

	^ self report: thisContext sender sender receiver! !

!LogCurrent methodsFor: 'public' stamp: 'test 12/3/2008 20:14'!
report: item

	^ LogReport new router: self; item: item; headerFor: item; yourself! !

!LogCurrent methodsFor: 'private' stamp: 'kph 5/12/2009 20:29'!
selectorAndArgs: aMessage

	| selector |
	selector := aMessage selector.
	selector isUnary ifTrue: [self << aMessage selector. ^ self endEntry].
	selector keywords with: aMessage arguments do: [:key :arg |
		self << key << ' ' <<  arg printString << ' '].
	self endEntry! !

!LogCurrent methodsFor: 'initialize-release' stamp: 'kph 10/8/2007 01:09'!
setPid: anInteger

	pid := anInteger! !

!LogCurrent methodsFor: 'accessing' stamp: 'mtf 4/29/2011 16:56'!
source
	^ source! !

!LogCurrent methodsFor: 'accessing' stamp: 'mtf 4/29/2011 16:56'!
source: anObject
	source := anObject! !

!LogCurrent methodsFor: 'public' stamp: 'mtf 4/4/2009 00:58'!
stack
"Log a stack trace"

"
This code was donated by Giovanni Giorgi (jj@objectsroot.com)  http://gioorgi.com/projects/
LogEngine and this snipped LogRouterAbstract-#stack is licenced MIT.
"
	"Call Example:
	stack==	an OrderedCollection('LogRouter>>buildStackTrace' '[] in LogClient>>privateBuildLineLimitedTo:usingID:obj:' 'String class(SequenceableCollection class)>>streamContents:limitedTo:' 'LogClient>>privateBuildLineLimitedTo:usingID:obj:' 'LogClient>>privateInfo:msg:' 'LogClient>>info:' '[] in LogTest class>>DoIt' '[] in BlockContext>>newProcess')"
	| stack |
	stack := LogException signal.	
	1 to: (stack size) do:[:i|
		('LogRouter(LogRouterAbstract)>>stack' = (stack at:i) )
		ifTrue:[ ^ self detailed: (stack copyFrom: i+2 to: stack size)]].

	^ self << 'No stack trace found??' ; endEntry! !

!LogCurrent methodsFor: 'levels' stamp: 'mtf 4/3/2009 23:43'!
temp
"Lowest priority: 11 of 11:
fatal panic emergency alert critical error warn notice info debug TEMP
This is the default level"

	Preferences enableTemporaryDebugMessages ifFalse: [ ^ self ignore ].

	category := nil.
	level := #temp! !

!LogCurrent methodsFor: 'levels' stamp: 'mtf 4/3/2009 23:58'!
temp: aString
"Logs aString at the temp level. Temp is level 11 of 11:
fatal panic emergency alert critical error warn notice info debug TEMP"

	self temp << aString ; endEntry! !

!LogCurrent methodsFor: 'public' stamp: 'mtf 4/4/2009 00:58'!
this
"Log the calling methods and its arguments"

 	self << sender receiver class << '-'.
	self selectorAndArgs: sender asMessage! !

!LogCurrent methodsFor: 'public' stamp: 'kph 12/1/2008 02:04'!
timeDelta: timerKey
	| past now key |
	
	now := TimeStamp now.
	key := timerKey asSymbol.
	past := self properties at: key ifAbsent: nil.
	self properties at: key  put: now.
	
	past ifNil: [ ^ self time start: key ].
	
	^ self time delta: now - past
	! !

!LogCurrent methodsFor: 'public' stamp: 'kph 11/29/2008 03:45'!
timeToRun: aBlock
	| time |
	[time := Time millisecondsToRun: aBlock]
		ensure: [self time: time] "use the dnu to output 'time: 1'"! !

!LogCurrent methodsFor: 'public' stamp: 'kph 12/1/2008 02:05'!
timeTotal: timerKey
	| past now key |
	
	now := TimeStamp now.
	key := timerKey asSymbol.
	past := self properties at: key ifAbsent: nil.
	
	past ifNil: [ self properties at: key put: now.
				^ self time start: key ].
	
	^ self time total: now - past
	! !

!LogCurrent methodsFor: 'public' stamp: 'mtf 4/24/2011 04:33'!
use: aBlock
"do aBlock if this logger is active"

	self ifActiveDo: aBlock! !

!LogCurrent methodsFor: 'levels' stamp: 'mtf 4/3/2009 23:38'!
warn
"Normal priority: 7 of 11:
fatal panic emergency alert critical error WARN notice info debug temp"

	level := #warn! !

!LogCurrent methodsFor: 'levels' stamp: 'mtf 4/3/2009 23:58'!
warn: aString
"Logs aString at the warn level. Warn is level 7 of 11:
fatal panic emergency alert critical error WARN notice info debug temp"

	self warn << aString ; endEntry! !

!LogCurrent methodsFor: 'line-buffer' stamp: 'mtf 1/27/2008 00:43'!
write 

	^ self endEntry! !

StandardFileStream subclass: #AppendFileStream
	instanceVariableNames: 'mutex'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Logging-Core'!

!AppendFileStream commentStamp: 'mtf 6/3/2011 15:52' prior: 0!
I am a file stream meant for write-only operation. I can be opened at once by multiple processes, and written concurrently. all operations should be done inside my critical block!

!AppendFileStream class methodsFor: 'file creation' stamp: 'mtf 6/3/2011 16:25'!
fileNamed: fileName
	| existingStream fullName |
	fullName := fileName asVmPathName.
	existingStream := Registry keys
		detect: [:file | file name sameAs: fullName]
		ifNone: [^ super fileNamed: fileName].
	(existingStream isKindOf: self) ifTrue: [^ existingStream].
	"If we reach here, the file  is already opened by an unsynchronized file stream. I'd throw an error, but the superclass method does not"
	^ nil! !

!AppendFileStream methodsFor: 'as yet unclassified' stamp: 'mtf 6/3/2011 16:39'!
critical: aBlock
	mutex ifNil: [mutex := Mutex new]. "initialize is never called; do it here"
	mutex critical: [
		self ensureOpen.
		self setToEnd.
		aBlock value.
		"self flush."
	]! !

SystemWindow subclass: #LogMorph
	instanceVariableNames: 'classes selectedClass selectedClasses selectedSeverity messages selectedMessage removedClasses searchString pauseResumeState bufferedMessages origLabel'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Logging-Adapters'!

LogMorph class
	instanceVariableNames: 'selected'!

!LogMorph commentStamp: 'gk 11/21/2007 09:11' prior: 0!
This is a simple but neat log emitter that shows log messages in a Morphic tool and offers filtering on level, sender and free text.
Open one up by:

   SLLogMorph openAndRegister!

LogMorph class
	instanceVariableNames: 'selected'!

!LogMorph class methodsFor: 'as yet unclassified' stamp: 'kph 12/2/2008 02:14'!
current
	
	^ self named: 'LogMorph'
! !

!LogMorph class methodsFor: 'as yet unclassified' stamp: 'kph 12/2/2008 02:41'!
named: aName
	
	| logs |
	
	logs := self windowsIn: self currentWorld satisfying: [ :w | w class = self and: [ w origLabel = aName ] ].
	
	logs ifEmpty: [ ^ nil ].

	^ logs first

! !

!LogMorph class methodsFor: 'as yet unclassified' stamp: 'kph 12/2/2008 02:57'!
open
	"Open in Morphic. Note that this does not register it
	as an emitter in any SLLog instance."

	^ self openNamed: 'LogMorph'! !

!LogMorph class methodsFor: 'as yet unclassified' stamp: 'kph 2/24/2009 16:48'!
openAndRegister
	"Open and register  standard singleton."

	LogCurrent adapters add: self open
	
! !

!LogMorph class methodsFor: 'as yet unclassified' stamp: 'kph 12/2/2008 02:06'!
openNamed: aName
	"Open in Morphic. Note that this does not register it
	as an emitter in any SLLog instance."

	"self open"

	^(self labelled: aName) open; yourself! !

!LogMorph methodsFor: 'accessing' stamp: 'makl 8/29/2006 17:23'!
bufferedMessages
	^bufferedMessages ifNil: [bufferedMessages := OrderedCollection new]! !

!LogMorph methodsFor: 'accessing' stamp: 'makl 8/29/2006 17:20'!
bufferedMessages: anObject
	bufferedMessages := anObject! !

!LogMorph methodsFor: 'menus' stamp: 'makl 8/11/2006 13:06'!
classMenu: aMenu
	"^ aMenu 
		labels: 'Do not log this class
Only log this class'
		lines: #()
		selections: #(removeClass onlyThisClass)"
	^ aMenu 
		labels: 'Do not log this class'
		lines: #()
		selections: #(removeClass)! !

!LogMorph methodsFor: 'accessing' stamp: 'makl 8/11/2006 07:51'!
classes
	^classes ifNil: [classes := SortedCollection new]! !

!LogMorph methodsFor: 'menus' stamp: 'makl 8/30/2006 13:20'!
clearMessages
	messages := OrderedCollection new.
	classes := OrderedCollection new.
	selectedClasses := nil.

	self changed: #selectedClass.
	self changed: #classes.	
	self changed: #selectedClasses.
	self changed: #filteredMessages
	! !

!LogMorph methodsFor: 'menus' stamp: 'makl 8/29/2006 16:58'!
clearToThisMessage
	(selectedMessage > 0) ifTrue: [
		messages := self messages copyFrom: selectedMessage to: messages size.
		"classes := messages inject: OrderedCollection new into: [:coll :msg |
					(coll includes: msg key) ifFalse: [coll add: msg key]. coll.]."
		self changed: #selectedClass.
		self changed: #classes.
		self changed: #filteredMessages
	].! !

!LogMorph methodsFor: 'accessing' stamp: 'kph 2/24/2009 16:37'!
critical: aBlock

		^ LogCurrent critical: aBlock! !

!LogMorph methodsFor: 'accessing' stamp: 'mtf 4/25/2011 01:11'!
filter
	^ nil! !

!LogMorph methodsFor: 'accessing' stamp: 'mtf 4/25/2011 01:12'!
filter: aValuable! !

!LogMorph methodsFor: 'private' stamp: 'kph 12/2/2008 03:12'!
filteredMessages
	self haltOnce.
	^ self messages inject: OrderedCollection new into: [:coll :each |
		((self testClass: each) and:
		[self testSeverity: each] and:
		[self testFilter: each])
		ifTrue: [coll add: each value value].
		coll
	]! !

!LogMorph methodsFor: 'private' stamp: 'kph 12/2/2008 03:17'!
getSeverity
	^#('0-emergency' '1-alert' '2-critical' '3-error' '4-warn' '5-notice' '6-info' '7-debug' '8-temp')! !

!LogMorph methodsFor: 'log protocol' stamp: 'mtf 4/29/2011 16:37'!
logEntry: aLogEntry formatted: aLogFormatter
"ignore the formatter for now"

	| str |
	
	(self removedClasses includes: aLogEntry sender name) ifFalse: [
		str := String new writeStream.
		
		str << aLogEntry stamp asTime print24 << ' - '<< aLogEntry level << ' '<< (aLogEntry category ifNil: ['']) << ': ' << aLogEntry line.
		
		self critical: [
			self pauseResumeState ifFalse: [
				self messages add: (aLogEntry sender name)->(aLogEntry numericLevel->str contents).
				(self classes anySatisfy: [:any | any = aLogEntry sender name]) ifFalse: [
					classes add: (aLogEntry sender name ifNil: [aLogEntry sender printString]).
					self changed: #classes].
				self changed: #filteredEntrys
			] ifTrue: [
				self bufferedMessages add: (aLogEntry sender name)->(aLogEntry numericLevel->str contents)
			]
		]
	]! !

!LogMorph methodsFor: 'accessing' stamp: 'makl 8/10/2006 19:17'!
messages
	^messages ifNil:[messages := OrderedCollection new]! !

!LogMorph methodsFor: 'menus' stamp: 'makl 8/29/2006 17:10'!
messagesMenu: aMenu
	"^ aMenu 
		labels: 'Do not log this class
Only log this class'
		lines: #()
		selections: #(removeClass onlyThisClass)"
	^ aMenu 
		labels: 'Clear all messages
Clear all messages before this one
', (self pauseResumeState ifTrue: ['Resume'] ifFalse: ['Pause'])
		lines: #(2)
		selections: #(clearMessages clearToThisMessage pauseResume)! !

!LogMorph methodsFor: 'menus' stamp: 'makl 8/11/2006 13:06'!
onlyThisClass
""! !

!LogMorph methodsFor: 'morphic' stamp: 'kph 12/2/2008 03:27'!
open
	"Open an instance in the World, but you need to register it
	as an emitter too."
	
	| horizontalSplit verticalSplit verticalSplit2 |
	horizontalSplit := 0.5.
	verticalSplit := 0.3.
	verticalSplit2 := 0.9.
	selectedClasses := IdentitySet new.
	self addMorph: (
		(PluggableListMorphOfMany
			on: self
			list: #classes
			primarySelection: #selectedClass
			changePrimarySelection: #selectedClass:
			listSelection: #selectedClassesAt:
			changeListSelection: #selectedClassesAt:put:
			menu: #classMenu:)	
	) frame: (0@0 corner: horizontalSplit@verticalSplit).
	
	self addMorph: (
		(PluggableListMorph
			on: self
			list: #getSeverity
			selected: #selectedSeverity
			changeSelected: #selectedSeverity:
			menu: nil)
		autoDeselect: true) frame: (horizontalSplit@0 corner: 1@verticalSplit).

	self addMorph: (
		(PluggableListMorph
			on: self
			list: #filteredMessages
			selected: #selectedMessage
			changeSelected: #selectedMessage:
			menu: #messagesMenu:)
		autoDeselect: false) frame: (0@verticalSplit corner: 1@verticalSplit2).

	self addMorph: (
			(PluggableTextMorph on: self 
				text: #searchString accept: #searchString:
				readSelection: nil menu: nil)
			acceptOnCR: true;
			setProperty: #alwaysAccept toValue: true;
			hideScrollBarsIndefinitely )
		frame: (0@verticalSplit2 corner: 1@1).

	self setWindowColor: (Color r: 0.75 g: 0.75 b: 0.05).
	self openInWorld.
	
	^self! !

!LogMorph methodsFor: 'morphic' stamp: 'kph 12/2/2008 03:11'!
openInWorld

	origLabel := self label.
	^ super openInWorld.! !

!LogMorph methodsFor: 'accessing' stamp: 'kph 12/2/2008 02:40'!
origLabel

	^ origLabel! !

!LogMorph methodsFor: 'menus' stamp: 'kph 12/2/2008 02:40'!
pauseResume
	(self pauseResumeState) ifFalse: [
		self setLabel: 'Paused'.
		pauseResumeState := true
	] ifTrue: [
		self setLabel: origLabel.
		pauseResumeState := false.
		messages addAll: self bufferedMessages.
		self bufferedMessages: OrderedCollection new.
		self changed: #classes.
		self changed: #filteredMessages
	]! !

!LogMorph methodsFor: 'accessing' stamp: 'makl 8/29/2006 17:08'!
pauseResumeState
	^pauseResumeState ifNil: [pauseResumeState := false].! !

!LogMorph methodsFor: 'accessing' stamp: 'makl 8/29/2006 17:02'!
pauseResumeState: anObject
	pauseResumeState := anObject! !

!LogMorph methodsFor: 'menus' stamp: 'makl 8/11/2006 12:33'!
perform: selector orSendTo: otherTarget 
	"Selector was just chosen from a menu by a user. If can respond, then  
	perform it on myself. If not, send it to otherTarget, presumably the  
	editPane from which the menu was invoked."
	(self respondsTo: selector)
		ifTrue: [^ self perform: selector]
		ifFalse: [^ super perform: selector orSendTo: otherTarget]! !

!LogMorph methodsFor: 'menus' stamp: 'makl 8/30/2006 13:19'!
removeClass
	selectedClass ifNotNil: [
		self removedClasses add: (selectedClass).
		messages := messages reject: [:e |
			e key = selectedClass].
		classes remove: selectedClass.
		self selectedClass: 0.
	
		self changed: #selectedClass.
		self changed: #classes.
		self changed: #selectedClasses.
		self changed: #filteredMessages
	]	! !

!LogMorph methodsFor: 'accessing' stamp: 'makl 8/11/2006 12:37'!
removedClasses
	^removedClasses ifNil:[removedClasses := Set new]! !

!LogMorph methodsFor: 'accessing' stamp: 'makl 8/21/2006 14:20'!
searchString
	^searchString ifNil: [searchString := '']! !

!LogMorph methodsFor: 'accessing' stamp: 'makl 8/21/2006 14:40'!
searchString: aString
	searchString := aString.
	self changed: #searchString.
	self changed: #filteredMessages! !

!LogMorph methodsFor: 'selected' stamp: 'makl 8/11/2006 08:02'!
selectedClass
	^classes ifNotNil:[classes indexOf: selectedClass] ifNil:[0]! !

!LogMorph methodsFor: 'selected' stamp: 'makl 8/11/2006 12:56'!
selectedClass: s
	selectedClass := ((s=0) or: [(classes at: s) = selectedClass])ifTrue: [nil] ifFalse: [classes at: s].
	self changed: #selectedClass.
	self changed: #filteredMessages! !

!LogMorph methodsFor: 'accessing' stamp: 'makl 8/30/2006 09:34'!
selectedClasses
	^selectedClasses ifNil: [selectedClasses := IdentitySet new]! !

!LogMorph methodsFor: 'accessing' stamp: 'makl 8/30/2006 09:16'!
selectedClasses: anObject
	selectedClasses := anObject! !

!LogMorph methodsFor: 'accessing' stamp: 'makl 8/30/2006 09:18'!
selectedClassesAt: index
	^ selectedClasses includes: index! !

!LogMorph methodsFor: 'accessing' stamp: 'makl 9/5/2006 10:50'!
selectedClassesAt:index put: value 
	value
		ifTrue: [selectedClasses add: index]
		ifFalse: [selectedClasses
				remove: index
				ifAbsent: []].
	self changed: #filteredMessages! !

!LogMorph methodsFor: 'selected' stamp: 'makl 8/11/2006 08:13'!
selectedMessage
	^selectedMessage ifNil:[selectedMessage := 0]! !

!LogMorph methodsFor: 'selected' stamp: 'makl 8/10/2006 19:00'!
selectedMessage: s
	selectedMessage := (s = selectedMessage) ifTrue: [nil] ifFalse: [s].
	self changed: #selectedMessage! !

!LogMorph methodsFor: 'selected' stamp: 'kph 12/2/2008 03:36'!
selectedSeverity
	^selectedSeverity ifNil: [ 11 ]! !

!LogMorph methodsFor: 'selected' stamp: 'kph 12/2/2008 03:36'!
selectedSeverity: s
	selectedSeverity := (s = 0 ifTrue: [ nil ] ifFalse: [ s ]).
	self changed: #selectedSeverity.
	self changed: #filteredMessages! !

!LogMorph methodsFor: 'testing' stamp: 'makl 9/5/2006 07:38'!
testClass: aMessage
	^(self selectedClasses anySatisfy: [:c | (classes at: c) = aMessage key]) or:
	[self selectedClasses isEmpty]! !

!LogMorph methodsFor: 'testing' stamp: 'FT 6/9/2009 14:02'!
testFilter: aMessage
	| lowerString cleanString |
	lowerString := aMessage value value asLowercase.
	^(self searchString asString substrings allSatisfy: [:sub |
		cleanString := (sub asLowercase) replaceAll: $_ with: (Character space).
		((cleanString first = $-) and: [(lowerString includesSubString: cleanString allButFirst) not]) or:
		[(cleanString first = $+) and: [lowerString includesSubString: cleanString allButFirst]] or:
		[lowerString includesSubString: cleanString]])! !

!LogMorph methodsFor: 'testing' stamp: 'kph 12/2/2008 03:35'!
testSeverity: aMessage
	 
	^(aMessage value key <= self selectedSeverity)! !

!LogMorph methodsFor: 'log protocol' stamp: 'mtf 4/29/2011 16:32'!
wouldLogEntry: aLogEntry
	^ true! !

Set subclass: #LogAdaptersSet
	instanceVariableNames: 'filter'
	classVariableNames: 'Protect'
	poolDictionaries: ''
	category: 'Logging-Core'!

!LogAdaptersSet commentStamp: 'kph 2/24/2009 14:42' prior: 0!
filterBlock

	self filterBlock: [ :level :category |  #(#error #debug #info) includes: level ]

example:

The transcript logs all data, the file logs only non temp messages.
	
thisLogInitialize: adapters

	^ adapters 
		addTranscript; 
		add: (LogAdaptersSet new 
				addFileLog: 'logs/', self class name, '.log';
				filterBlock: [ :l :c | l ~= #temp ];
				yourself);
		yourself
					!

!LogAdaptersSet class methodsFor: 'as yet unclassified' stamp: 'kph 12/1/2008 21:13'!
cleanUp

	self initialize! !

!LogAdaptersSet class methodsFor: 'as yet unclassified' stamp: 'kph 12/1/2008 22:00'!
initialize

	Protect := Semaphore forMutualExclusion.! !

!LogAdaptersSet methodsFor: 'as yet unclassified' stamp: 'kph 2/24/2009 16:47'!
add: anAdapter
	(self find: LogHistory) ifNotNilDo: [ :history | 

		anAdapter value ifNotNilDo: [ :a | history outputTo: a ].
		
		"dont add a second history adapter"
		anAdapter class = LogHistory ifTrue: [ ^ self ] 
	].
	
	(self find: Transcript) ifNotNil: [ ^ self ].
	
	Protect critical: [ super add: anAdapter ].

	^ anAdapter! !

!LogAdaptersSet methodsFor: 'accessing' stamp: 'kph 2/24/2009 02:07'!
addHistoryLog
	 
	^ self add: (LogHistory size: 30)! !

!LogAdaptersSet methodsFor: 'as yet unclassified' stamp: 'kph 2/24/2009 02:08'!
addTranscript
	 
	(self includes: Transcript) ifFalse: [ self add: Transcript ]! !

!LogAdaptersSet methodsFor: 'as yet unclassified' stamp: 'kph 12/2/2008 02:53'!
do: aBlock 

	"should cope better than do: if another thread adds or removes items"
 
	tally = 0 ifTrue: [^ self].
	1 to: array size do:
		[:index |
		| each |
		(each := array at: index ifAbsent: [ ^ self ]) ifNotNil: [aBlock value: each]]! !

!LogAdaptersSet methodsFor: 'accessing' stamp: 'mtf 4/25/2011 00:58'!
filter
	^ filter! !

!LogAdaptersSet methodsFor: 'accessing' stamp: 'mtf 4/25/2011 00:59'!
filter: aValuable
	filter := aValuable! !

!LogAdaptersSet methodsFor: 'as yet unclassified' stamp: 'kph 2/24/2009 16:45'!
find: aClass
	 
	^ self detect: [ :ea | ea class = aClass ] ifNone: [ nil ]! !

!LogAdaptersSet methodsFor: 'as yet unclassified' stamp: 'mtf 4/29/2011 16:36'!
logEntry: aLogEntry formatted: aLogFormatter

	filter ifNotNil: [(filter value: aLogEntry) ifFalse: [^ self]].

	self do: [ :ea | 
		ea value ifNotNilDo: [ :log | 
			log logEntry: aLogEntry formatted: aLogFormatter
		]
	]! !

!LogAdaptersSet methodsFor: 'as yet unclassified' stamp: 'kph 3/5/2009 13:32'!
remove: anAdapter

	Protect critical: [ super remove: anAdapter ifAbsent: [] ].
! !

!LogAdaptersSet methodsFor: 'as yet unclassified' stamp: 'kph 12/1/2008 04:19'!
removeAll

	Protect critical: [ self initialize: 5 ].

! !

!LogAdaptersSet methodsFor: 'as yet unclassified' stamp: 'mtf 4/29/2011 16:32'!
wouldLogEntry: aLogEntry
	filter ifNotNil: [(filter value: aLogEntry) ifFalse: [^ false]].
	^ self anySatisfy: [:ea | ea wouldLogEntry: aLogEntry]! !

TestCase subclass: #LogTests
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Logging-Tests'!

!LogTests commentStamp: 'gk 6/1/2007 13:20' prior: 0!
A few simple tests.!

!LogTests methodsFor: 'as yet unclassified' stamp: 'kph 2/24/2009 17:47'!
testLogFile
	
	"
	self debug: #testLogFile.
	"
	| levels all some crlf |

	LogCurrent value: LogCurrent new. 
	


	crlf := String with: Character cr with: Character lf.
	levels := {#emergency. #alert. #critical. #error". #warning. #notice. #info. #debug"}.
	FileDirectory default deleteFileNamed: 'testall.log'; deleteFileNamed: 'testsome.log'.
	all := (LogFile named: 'testall.log') output: [:entry |
			entry level , ': ', entry line, ' (', entry sender methodClass name, ')'].

	some := (LogFile named: 'testsome.log') output: [:entry |
				((levels includes: entry level) and: 
				[entry line beginsWith: 'makl'] and:
				[entry sender notNil]) ifTrue: [
					entry level, ': ', entry line, ' (', entry sender methodClass name , ')'
				]
			].
		
	LogCurrent adapters add: all; add: some.	
		
	"Do some stuff"
	self log info: 'Testing info'.
	self log warn: 'Testing warning'.
	self log error: 'makl cannot code!!'.
	
	LogCurrent value: nil.

	all := (FileDirectory default) oldFileNamed: 'testall.log'.
	some := (FileDirectory default) oldFileNamed: 'testsome.log'.
	self assert: (all contents  = 	('---------------------------', crlf,
								'info: Testing info (LogTests)', crlf, 
								'warn: Testing warning (LogTests)', crlf, 
								'error: makl cannot code!! (LogTests)', crlf)).
	self assert: (some contents = 	('---------------------------', crlf,
								'error: makl cannot code!! (LogTests)', crlf)).
	all close.
	some close.

	FileDirectory default deleteFileNamed: 'testall.log'; deleteFileNamed: 'testsome.log'.! !

TestCase subclass: #LoggingTest
	instanceVariableNames: 'log'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Logging-Tests'!

!LoggingTest class methodsFor: 'regression-tests' stamp: 'kph 11/26/2008 05:29'!
LogClient

	^ Smalltalk at: #LogClient ifAbsent: [ self error: 'LogEngine not loaded' ]! !

!LoggingTest class methodsFor: 'regression-tests' stamp: 'kph 11/26/2008 05:30'!
test1
	"LogTest test1"
	| lc lc2 |
	self LogClient reset.
	lc := LogClient for: 'RegressionTest1'.
	lc2 := LogClient for: 'RegressionTest2'.
	lc2 debug: 'test2 Printed first'.
	lc debug: 'test1 Printed for second'.
	lc info: 'You can also print lines where code is called!!'.
	lc info: ' Info level'.
	lc error: 'Error level'.
	self test2.
	LogClient reset! !

!LoggingTest class methodsFor: 'regression-tests' stamp: 'kph 11/26/2008 05:30'!
test2
	| lc |
	lc := LogClient for: 'RegressionTest1'.
     lc info: 'You can also print lines where code is called and this is a different line'.
! !

!LoggingTest class methodsFor: 'regression-tests' stamp: 'kph 11/26/2008 05:30'!
test3
	"Multi fork really rocks!! This tests checks your concurrecy speed. If you get something like:
26.Aug.2003-23:29:23 <I> ShockTest 'Trying to crash me1'
26.Aug.2003-23:29:23 !!!!!!  <I> ShockTest 'Trying to crash me2'
26.Aug.2003-23:29:23 !!!!!!  <I> ShockTest 'Trying to crash me3'
26.Aug.2003-23:29:23 !!!!!!  <I> ShockTest 'Trying to crash me4'
26.Aug.2003-23:29:23 !!!!!!  <I> ShockTest 'Trying to crash me5'
26.Aug.2003-23:29:24 !!!!!!  <I> ShockTest 'Trying to crash me6'
26.Aug.2003-23:29:24 <I> ShockTest 'Trying to crash me7'
26.Aug.2003-23:29:25 <I> ShockTest 'Trying to crash me8'

your computer is ok
If you get
26.Aug.2003-23:29:23 <I> ShockTest 'Trying to crash me1'
26.Aug.2003-23:29:23 !!!!!!  <I> ShockTest 'Trying to crash me2'
26.Aug.2003-23:29:23 !!!!!!  <I> ShockTest 'Trying to crash me3'
26.Aug.2003-23:29:23 !!!!!!  <I> ShockTest 'Trying to crash me4'
26.Aug.2003-23:29:23 !!!!!!  <I> ShockTest 'Trying to crash me5'
26.Aug.2003-23:29:24 !!!!!!  <I> ShockTest 'Trying to crash me6'
26.Aug.2003-23:29:24 !!!!!!  <I> ShockTest 'Trying to crash me7'
26.Aug.2003-23:29:25 !!!!!!  <I> ShockTest 'Trying to crash me8'
Your computer is slow and some race condition can happen!!
	"
	| lc |
	lc := LogClient for: 'ShockTest'.

	[ lc info:'Trying to crash me1'. ] fork.		
	[ lc info:'Trying to crash me2'. ] fork.	
	(Delay forMilliseconds:40	) wait.	
	[ lc info:'Trying to crash me3'. ] fork.		
	(Delay forMilliseconds:80	) wait.	
	[ lc info:'Trying to crash me4'. ] fork.		
	(Delay forMilliseconds:160	) wait.	
	[ lc info:'Trying to crash me5'. ] fork.	
	(Delay forMilliseconds:340	) wait.
	[ lc info:'Trying to crash me6'. ] fork.	
	(Delay forMilliseconds:440	) wait.
	[ lc info:'Trying to crash me7'. ] fork.	
	(Delay forMilliseconds:840	) wait.
	[ lc info:'Trying to crash me8'. ] fork.	! !

!LoggingTest class methodsFor: 'regression-tests' stamp: 'kph 11/26/2008 05:30'!
test4
	"Multi fork really rocks!!"
	| lc |
	lc := LogClient for: 'ShockTest'.

	[ lc info:'Trying to crash me1'. ] fork.		
	[ lc info:'Trying to crash me2'. ] fork.		
	[ lc info:'Trying to crash me3'. ] fork.		
	[ lc info:'Trying to crash me4'. ] fork.		
	[ lc info:'Trying to crash me5'. ] fork.	
	(Delay forMilliseconds:340	) wait.
	[ lc info:'Trying to crash me6'. ] fork.	
	(Delay forMilliseconds:440	) wait.
	[ lc info:'Trying to crash me7'. ] fork.	
	(Delay forMilliseconds:840	) wait.
	[ lc info:'Trying to crash me8'. ] fork.	! !

!LoggingTest methodsFor: 'as yet unclassified' stamp: 'kph 11/27/2008 22:50'!
acceptance

	self log error test: 1.
	self log this.
	self log test: 3.
	self log myown test: 4.
	self log error myown  test: 5.

	^ self output	
"
self new acceptance.
"  ! !

!LoggingTest methodsFor: 'as yet unclassified' stamp: 'kph 11/27/2008 03:18'!
acceptance: a

	self log error test: 1.
	self log this.
	self log debug this.
	self log test: 3.
	self log myown test: 4.
	self log error myown  test: 5.
	self log it all.
	
	^ self output	
 ! !

!LoggingTest methodsFor: 'as yet unclassified' stamp: 'kph 11/26/2008 06:39'!
acceptanceStack

	self log stack.

	^ self output	
"
self new acceptanceStack.
"  ! !

!LoggingTest methodsFor: 'as yet unclassified' stamp: 'kph 10/8/2007 01:24'!
assertOutput: string
	
	^ self assert: (self output = string)! !

!LoggingTest methodsFor: 'as yet unclassified' stamp: 'kph 12/3/2008 20:02'!
log

	"we simulate the idea that a specific log adaptor is instanciated once per thread"
	
	^ (log ifNil: [ log := LogCurrent new adapter: LogTestAdapter new ])
		inSender: thisContext sender 
	
  ! !

!LoggingTest methodsFor: 'as yet unclassified' stamp: 'kph 12/3/2008 20:01'!
output

	^ log adapter output
  ! !

!LoggingTest methodsFor: 'as yet unclassified' stamp: 'kph 11/26/2008 05:27'!
speedNull
	| out  time |
	
	out := WriteStream on: (String new: 100).
	time  := [100000 timesRepeat: [
	out reset.
	(null << 'hello' << 'test' << 'the' << 'world') "contents do: [ :each | out nextPutAll: each ]"
	]] timeToRun.

	^ time 

	 ! !

!LoggingTest methodsFor: 'as yet unclassified' stamp: 'kph 10/7/2007 21:34'!
speedTest
	
	"not a test but code for making speed comparisons.
	the SUnit test runners do not 'yet' have performance reporting
	speed* seems like a sensible prefix to use for this"
	
	| out  time |
	
	out := WriteStream on: (String new: 100).
	
	time  := [100000 timesRepeat: [
	out reset.
	out nextPutAll: ('hello' , 'test' , 'the', 'world')
	]] timeToRun.

	^ time! !

!LoggingTest methodsFor: 'as yet unclassified' stamp: 'kph 10/7/2007 21:31'!
speedTest2
	| out  time |
	
	out := WriteStream on: (String new: 100).
	
	time  := [100000 timesRepeat: [
	out reset.
	out nextPutAll: 'hello'.
	out nextPutAll: 'test'.
	out nextPutAll: 'the'.
	out nextPutAll: 'world'
	]] timeToRun.

	time
	! !

!LoggingTest methodsFor: 'as yet unclassified' stamp: 'kph 10/7/2007 21:32'!
speedTest3
	| out  time |
	
	out := WriteStream on: (String new: 100).
	
	time  := [100000 timesRepeat: [
	out reset.
	 #('hello' 'test' 'the' 'world') do: [ :each | out nextPutAll: each ]
	]] timeToRun.

	time! !

!LoggingTest methodsFor: 'as yet unclassified' stamp: 'kph 2/24/2009 17:25'!
testAcceptance1
	"
	self debug: #testAcceptance1
	"

 	self assert: (self acceptance = 'ERROR LOGGINGTEST test: 1 
TEMP LOGGINGTEST LoggingTest-acceptance
TEMP LOGGINGTEST test: 3 
TEMP MYOWN test: 4 
ERROR MYOWN test: 5 
')! !

!LoggingTest methodsFor: 'as yet unclassified' stamp: 'kph 2/24/2009 17:26'!
testAcceptance2
	"
	self debug: #testAcceptance2
	"

 	self assert: ((self acceptance: 2) beginsWith: 'ERROR LOGGINGTEST test: 1 
TEMP LOGGINGTEST LoggingTest-acceptance: 2 
DEBUG LOGGINGTEST LoggingTest-acceptance: 2 
TEMP LOGGINGTEST test: 3 
TEMP MYOWN test: 4 
ERROR MYOWN test: 5 
TEMP LOGGINGTEST LoggingTest>>#testAcceptance2 {as yet unclassified}
category: ''Instance'' 
TEMP LOGGINGTEST NAMED SLOTS
testSelector: testAcceptance2
log: a LogCurrent

TEMP LOGGINGTEST category: ''Class'' 
TEMP LOGGINGTEST LoggingTest
CLASS NAMED SLOTS
superclass: TestCase
methodDict: a MethodDictionary(')! !

!LoggingTest methodsFor: 'as yet unclassified' stamp: 'kph 2/24/2009 17:28'!
testAnnotated

"
self debug: #testAnnotated.
"

	self log info x: 10 y: 20 pointName: 'corner'.
	
	self assertOutput:'INFO LOGGINGTEST x: 10 y: 20 pointName: ''corner'' 
'.
	! !

!LoggingTest methodsFor: 'as yet unclassified' stamp: 'kph 10/8/2007 05:28'!
testDynamicCategory

"
self debug: #testDynamicCategory.
"

	self log info database <<< 'query'.
	
	self log database info <<< 'query'.
	 ! !

!LoggingTest methodsFor: 'as yet unclassified' stamp: 'test 2/24/2009 17:32'!
testPrivateViolation 
	"
	self debug: #testPrivateViolation
	"

	self log adapter privateMethodFor: self.

 	self assert: (self output = 'WARN LOGTESTADAPTER PRIVATE VIOLATION
') 
    ! !

!LoggingTest methodsFor: 'as yet unclassified' stamp: 'kph 12/3/2008 20:15'!
testPrivateViolationNot
	"
	self debug: #testPrivateViolation
	"
 
	self log privateViolation.

 	self assert: (self output = '') 
    ! !

!LoggingTest methodsFor: 'as yet unclassified' stamp: 'kph 2/24/2009 17:33'!
testThis

"
self debug: #testThis.
"

	self x: 10 y: 20 pointName: 'corner'.
	
	self assertOutput: 'TEMP LOGGINGTEST LoggingTest-x: 10 y: 20 pointName: ''corner'' 
'.
! !

!LoggingTest methodsFor: 'as yet unclassified' stamp: 'kph 10/8/2007 08:27'!
x: x y: y  pointName: it
	

	self log this	! !

!TranscriptStream methodsFor: '*logging' stamp: 'mtf 4/25/2011 01:14'!
filter
	^ nil! !

!TranscriptStream methodsFor: '*logging' stamp: 'mtf 4/25/2011 01:14'!
filter: aValuable! !

!TranscriptStream methodsFor: '*logging' stamp: 'mtf 5/3/2011 20:09'!
logEntry: aLogEntry

 	self semaphore critical:[

		self << aLogEntry stamp asTime print24 << '[' << aLogEntry pid << '] ' << (aLogEntry category ifNil: [ aLogEntry level ]) << ' ' << aLogEntry line; cr.
		self changed: #appendEntry.
		self reset.

 	].! !

!TranscriptStream methodsFor: '*logging' stamp: 'mtf 4/29/2011 16:33'!
wouldLogEntry: aLogEntry
	^ true! !
LogCurrent initialize!
LogAdaptersSet initialize!
